@@ -2,7 +2,7 @@
  *
  * General channel type for forwarding data to an fd
  *
- * $Id: channel_forward.c,v 1.2 2001/08/30 21:02:06 nisse Exp $ */
+ * $Id: channel_forward.c,v 1.3 2001/09/21 13:37:16 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -97,7 +97,7 @@ do_channel_forward_send_adjust(struct ssh_channel *s,
 {
   CAST(channel_forward, self, s);
   
-  self->socket->want_read = 1;
+  lsh_oop_register_read_fd(self->socket);
 }
 
 static void
