@@ -1,6 +1,6 @@
 /* client.c
  *
- * $Id: client.c,v 1.129 2001/09/04 13:23:23 nisse Exp $ */
+ * $Id: client.c,v 1.134 2001/10/03 13:42:38 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -76,10 +76,7 @@ format_service_request(int name)
      (super packet_handler)
      (vars
        (service . int)
-       (c object command_continuation)
-
-       ;; FIXME: Do we really need the exception handler here?
-       (e object exception_handler)))
+       (c object command_continuation)))
 */
 
 static void
@@ -112,20 +109,18 @@ do_accept_service(struct packet_handler *c,
       COMMAND_RETURN(closure->c, connection);
     }
   else
-    PROTOCOL_ERROR(closure->e, "Invalid SSH_MSG_SERVICE_ACCEPT message");
+    PROTOCOL_ERROR(connection->e, "Invalid SSH_MSG_SERVICE_ACCEPT message");
 }
 
 struct packet_handler *
 make_accept_service_handler(UINT32 service,
-			    struct command_continuation *c,
-			    struct exception_handler *e)
+			    struct command_continuation *c)
 {
   NEW(accept_service_handler, closure);
 
   closure->super.handler = do_accept_service;
   closure->service = service;
   closure->c = c;
-  closure->e = e;
   
   return &closure->super;
 }
@@ -134,13 +129,15 @@ void
 do_request_service(struct command *s,
 		   struct lsh_object *x,
 		   struct command_continuation *c,
-		   struct exception_handler *e)
+		   struct exception_handler *e UNUSED)
 {
   CAST(request_service, self, s);
   CAST(ssh_connection, connection, x);
-  
+
+  /* NOTE: Uses the connection's exception handler, not the one passed
+   * in. */
   connection->dispatch[SSH_MSG_SERVICE_ACCEPT]
-    = make_accept_service_handler(self->service, c, e);
+    = make_accept_service_handler(self->service, c);
   
   C_WRITE(connection,
 	  format_service_request(self->service));
@@ -382,15 +379,13 @@ make_client_session(struct client_options *options);
 
 void
 init_client_options(struct client_options *self,
-		    struct io_backend *backend,
 		    struct randomness_with_poll *random,
 		    struct exception_handler *handler,
 		    int *exit_code)			 
 {
-  self->backend = backend;
   self->random = random;
   
-  self->tty = make_unix_interact(backend);
+  self->tty = make_unix_interact();
   self->escape = -1;
   
   self->handler = handler;
@@ -496,10 +491,6 @@ client_options[] =
   { "no-cvs-workaround", OPT_FORK_STDIO | ARG_NOT, NULL, 0,
     "Disable the cvs workaround.", 0 },
 
-  /* FIXME: Perhaps this should be moved from client.c to lsh.c? It
-   * doesn't work with lshg. Or perhaps that can be fixed?
-   * About the same problem applies to -R. */
-
 #if WITH_PTY_SUPPORT
   { "pty", 't', NULL, 0, "Request a remote pty (default).", 0 },
   { "no-pty", 't' | ARG_NOT, NULL, 0, "Don't request a remote pty.", 0 },
@@ -601,9 +592,9 @@ client_shell_session(struct client_options *options)
   if (session)
     {
       struct object_queue session_requests;
-    
+
       object_queue_init(&session_requests);
-  
+
       client_maybe_pty(options, &session_requests);
       client_maybe_x11(options, &session_requests);
   
@@ -825,6 +816,7 @@ make_client_session(struct client_options *options)
   int out;
   int err;
   int is_tty = 0;
+  struct ssh_channel *session;
   
   struct escape_info *escape = NULL;
   
@@ -918,20 +910,24 @@ make_client_session(struct client_options *options)
 
   /* Clear options */
   options->stdin_file = options->stdout_file = options->stderr_file = NULL;
-  
-  return make_client_session_channel
-    (io_read(make_lsh_fd(options->backend,
-			 in, "client stdin", options->handler),
+
+  session = make_client_session_channel
+    (io_read(make_lsh_fd(in, "client stdin", options->handler),
 	     NULL, NULL),
-     io_write(make_lsh_fd(options->backend,
-			  out, "client stdout", options->handler),
+     io_write(make_lsh_fd(out, "client stdout", options->handler),
 	      BLOCK_SIZE, NULL),
-     io_write(make_lsh_fd(options->backend,
-			  err, "client stderr", options->handler),
+     io_write(make_lsh_fd(err, "client stderr", options->handler),
 	      BLOCK_SIZE, NULL),
      escape,
      WINDOW_SIZE,
      options->exit_code);
+
+  /* The channel won't get registered in any other resource_list
+   * until later, so we must register it here to avoid a "garbage
+   * collecting a live resource!" crashes if the connection fails
+   * early. */
+  gc_global(&session->resources->super);
+  return session;
 }
 
 
@@ -1063,7 +1059,7 @@ client_argp_parser(int key, char *arg, struct argp_state *state)
 	client_add_action(options,
 			  make_install_fix_channel_open_handler
 			  (ATOM_X11,
-			   make_channel_open_x11(options->backend)));
+			   &channel_open_x11));
       
       /* Install suspend-handler */
       suspend_install_handler();
@@ -1104,8 +1100,7 @@ client_argp_parser(int key, char *arg, struct argp_state *state)
 	  argp_error(state, "Invalid forward specification '%s'.", arg);
 
 	client_add_action(options, make_forward_local_port
-			  (options->backend,
-			   make_address_info((options->with_remote_peers
+			  (make_address_info((options->with_remote_peers
 					      ? NULL
 					      : ssh_format("%lz", "127.0.0.1")),
 					     listen_port),
@@ -1113,26 +1108,6 @@ client_argp_parser(int key, char *arg, struct argp_state *state)
 	break;
       }      
 
-#if 0
-    case 'R':
-      {
-	UINT32 listen_port;
-	struct address_info *target;
-
-	if (!client_parse_forward_arg(arg, &listen_port, &target))
-	  argp_error(state, "Invalid forward specification '%s'.", arg);
-
-	client_add_action(options, make_forward_remote_port
-			  (options->backend,
-			   make_address_info((options->with_remote_peers
-					      ? ssh_format("%lz", "0.0.0.0")
-					      : ssh_format("%lz", "127.0.0.1")),
-					     listen_port),
-			   target));
-	options->remote_forward = 1;
-	break;
-      }      
-#endif
     case 'N':
       options->start_shell = 0;
       break;
