@@ -2,7 +2,7 @@
  *
  *
  *
- * $Id: client.h,v 1.44 2001/09/01 22:59:29 nisse Exp $ */
+ * $Id: client.h,v 1.47 2001/09/27 06:33:10 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -40,8 +40,7 @@
 
 struct packet_handler *
 make_accept_service_handler(UINT32 service,
-			    struct command_continuation *c,
-			    struct exception_handler *e);
+			    struct command_continuation *c);
 
 /* GABA:
    (class
@@ -101,9 +100,7 @@ make_exec_request(struct lsh_string *command);
 struct command *
 make_pty_request(struct interact *tty);
 
-
-struct channel_open *
-make_channel_open_x11(struct io_backend *backend);
+extern struct channel_open channel_open_x11;
 
 struct command *
 make_forward_x11(const char *display_string,
@@ -116,8 +113,6 @@ make_client_x11_display(const char *display, struct lsh_string *fake);
    (class
      (name client_options)
      (vars
-       (backend object io_backend)
-
        ;; Used only by lsh, NULL for lshg.
        (random object randomness_with_poll)
 
@@ -168,7 +163,6 @@ make_client_x11_display(const char *display, struct lsh_string *fake);
 
 void
 init_client_options(struct client_options *options,
-		    struct io_backend *backend,
 		    struct randomness_with_poll *random,
 		    struct exception_handler *handler,
 		    int *exit_code);
