@@ -1,6 +1,6 @@
 /* client_session.c
  *
- * $Id: client_session.c,v 1.10 2001/06/19 21:18:55 nisse Exp $
+ * $Id: client_session.c,v 1.13 2001/10/03 13:42:33 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -55,7 +55,6 @@
 
 /* Callback used when the server sends us eof */
 
-/* FIXME: Perhaps unify with server_session.c:do_eof */
 static void
 do_client_session_eof(struct ssh_channel *c)
 {
@@ -104,7 +103,7 @@ do_send_adjust(struct ssh_channel *s,
 
   assert(self->in->read);
 
-  self->in->want_read = 1;
+  lsh_oop_register_read_fd(self->in);
 }
 
 /* Escape char handling */
@@ -136,10 +135,10 @@ do_client_io(struct command *s UNUSED,
   
   channel->receive = do_receive;
 
-  /* FIXME: It seems a little kludgy to modify
-   * exception handlers here; it would be better to create the
-   * fd-objects at a point where the right exception handlers can be
-   * installed from the start. */
+  /* FIXME: It seems a little kludgy to modify exception handlers
+   * here; it would be better to create the fd-objects at a point
+   * where the right exception handlers can be installed from the
+   * start. */
   session->out->e
     = make_channel_io_exception_handler(channel,
 					"lsh: I/O error on stdout",
@@ -161,7 +160,7 @@ do_client_io(struct command *s UNUSED,
   /* FIXME: Perhaps there is some way to arrange that channel.c calls
    * the CHANNEL_SEND_ADJUST method instead? */
   if (session->super.send_window_size)
-    session->in->want_read = 1;
+    lsh_oop_register_read_fd(session->in);
   
   session->in->close_callback
     = make_channel_read_close_callback(channel);
@@ -195,6 +194,7 @@ make_client_session_channel(struct lsh_fd *in,
 {
   NEW(client_session_channel, self);
 
+  trace("make_client_session_channel\n");
   init_channel(&self->super);
 
   /* Makes sure the pending_close bit is set whenever this session
