@@ -2,7 +2,7 @@
  *
  * Client side of X11 forwaarding.
  *
- * $Id: client_x11.c,v 1.11 2001/09/03 21:38:46 nisse Exp $ */
+ * $Id: client_x11.c,v 1.15 2001/09/27 06:34:59 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -63,7 +63,7 @@
        (auth string)))
 */
 
-/* FIXME: Perhaps merge auth_info directly into this struct */
+/* FIXME: Perhaps merge auth_info directly into this struct. */
 /* GABA:
    (class
      (name client_x11_display)
@@ -423,16 +423,14 @@ make_exc_x11_connect_handler(struct exception_handler *parent,
   return make_exception_handler(do_exc_x11_connect_handler, parent, context);
 }
 
-static void
-do_channel_open_x11(struct channel_open *s,
-		    struct ssh_connection *connection,
-		    struct channel_open_info *info UNUSED,
-		    struct simple_buffer *args,
-		    struct command_continuation *c,
-		    struct exception_handler *e)
+DEFINE_CHANNEL_OPEN(channel_open_x11)
+     (struct channel_open *s UNUSED,
+      struct ssh_connection *connection,
+      struct channel_open_info *info UNUSED,
+      struct simple_buffer *args,
+      struct command_continuation *c,
+      struct exception_handler *e)
 {
-  CAST(channel_open_x11, self, s);
-
   UINT32 originator_length;
   const UINT8 *originator;
   UINT32 originator_port;
@@ -450,8 +448,7 @@ do_channel_open_x11(struct channel_open *s,
       if (display)
 	{
 	  struct lsh_fd *fd
-	    = io_connect(self->backend,
-			 display->address,
+	    = io_connect(display->address,
 			 display->address_length,
 			 make_channel_open_x11_continuation(display,
 							    c),
@@ -479,17 +476,6 @@ do_channel_open_x11(struct channel_open *s,
     }
 }
 
-struct channel_open *
-make_channel_open_x11(struct io_backend *backend)
-{
-  NEW(channel_open_x11, self);
-
-  self->super.handler = do_channel_open_x11;
-  self->backend = backend;
-
-  return &self->super;
-}
-
 
 /* Format is host:display.screen, where display and screen are numbers */
 static int
@@ -524,9 +510,9 @@ parse_display(struct client_x11_display *self, const char *display)
   
   /* Get display number */
   {
+    char *end;
     num = display;
     
-    char *end;
     display_num = strtol(display, &end, 0);
 
     num_length = end - num;
