@@ -205,6 +205,9 @@ do_exc_connection_handler(struct exception_handler *s,
     case EXC_PROTOCOL:
       {
 	CAST_SUBTYPE(protocol_exception, exc, e);
+
+        werror("Protocol error: %z\n", e->msg);
+        
 	if (exc->reason)
 	  C_WRITE(self->connection, format_disconnect(exc->reason, exc->super.msg, ""));
 	
@@ -289,7 +292,7 @@ make_ssh_connection(UINT32 flags,
   connection->peer_flags = 0;
   connection->user = NULL;
   
-  connection->resources = empty_resource_list();
+  connection->resources = make_resource_list();
   
   connection->rec_max_packet = SSH_MAX_PACKET;
 
