@@ -2,7 +2,7 @@
  *
  *
  *
- * $Id: crypto.h,v 1.22 2001/04/26 04:41:42 nisse Exp $ */
+ * $Id: crypto.h,v 1.29 2001/10/15 12:40:38 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -35,34 +35,13 @@
 		  (src) += (blocksize),		\
 		  (dst) += (blocksize)) )
 
+extern struct crypto_algorithm crypto_aes256_cbc_algorithm;
 extern struct crypto_algorithm crypto_arcfour_algorithm;
-extern struct crypto_algorithm crypto_des_algorithm;
-struct crypto_algorithm *make_des3(void);
-
-struct crypto_algorithm *make_cast_algorithm(UINT32 key_size);
-extern struct crypto_algorithm cast128_algorithm;
-
-struct crypto_algorithm *make_blowfish_algorithm(UINT32 key_size);
-extern struct crypto_algorithm blowfish_algorithm;
-
-struct crypto_algorithm *make_twofish_algorithm(UINT32 key_size);
-extern struct crypto_algorithm twofish128_algorithm;
-extern struct crypto_algorithm twofish192_algorithm;
-extern struct crypto_algorithm twofish256_algorithm;
-
-struct crypto_algorithm *make_rijndael_algorithm(UINT32 key_size);
-extern struct crypto_algorithm rijndael128_algorithm;
-extern struct crypto_algorithm rijndael192_algorithm;
-extern struct crypto_algorithm rijndael256_algorithm;
-
-struct crypto_algorithm *make_serpent_algorithm(UINT32 key_size);
-extern struct crypto_algorithm serpent128_algorithm;
-extern struct crypto_algorithm serpent192_algorithm;
-extern struct crypto_algorithm serpent256_algorithm;
-
-#ifdef WITH_IDEA
-extern struct crypto_algorithm idea_algorithm;
-#endif
+extern struct crypto_algorithm crypto_blowfish_cbc_algorithm;
+extern struct crypto_algorithm crypto_cast128_cbc_algorithm;
+extern struct crypto_algorithm crypto_des3_cbc_algorithm;
+extern struct crypto_algorithm crypto_serpent256_cbc_algorithm;
+extern struct crypto_algorithm crypto_twofish256_cbc_algorithm;
 
 extern struct hash_algorithm sha1_algorithm;
 extern struct hash_algorithm md5_algorithm;
