@@ -1,6 +1,6 @@
 /* dsa.c
  *
- * $Id: dsa.c,v 1.28 2001/06/11 23:06:03 nisse Exp $
+ * $Id: dsa.c,v 1.30 2001/10/08 15:37:29 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -34,7 +34,7 @@
 #include "werror.h"
 #include "xalloc.h"
 
-#include "nettle/sha1.h"
+#include "nettle/sha.h"
 
 #include <assert.h>
 
@@ -526,7 +526,8 @@ do_dsa_sign(struct signer *c,
       
       break;
     default:
-      fatal("do_dsa_sign: Internal error.");
+      fatal("do_dsa_sign: Internal error, unexpected algorithm %a.\n",
+	    algorithm);
     }
   mpz_clear(r);
   mpz_clear(s);
