@@ -2,7 +2,7 @@
  *
  * Interact with the user.
  *
- * $Id: interact.h,v 1.8 2001/04/26 04:46:04 nisse Exp $*/
+ * $Id: interact.h,v 1.9 2001/09/24 16:51:10 nisse Exp $*/
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -98,6 +98,6 @@ struct terminal_dimensions;
   ((i)->window_change_subscribe((i), (c)))
 
 struct interact *
-make_unix_interact(struct io_backend *backend);
+make_unix_interact(void);
 
 #endif /* LSH_INTERACT_H_INCLUDED */
