@@ -1,6 +1,6 @@
 /* io.c
  *
- * $Id: io.c,v 1.140 2001/09/01 10:43:38 nisse Exp $ */
+ * $Id: io.c,v 1.155 2001/10/02 16:15:23 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -24,6 +24,7 @@
 #include "io.h"
 
 #include "format.h"
+#include "string_buffer.h"
 #include "werror.h"
 #include "xalloc.h"
 
@@ -44,6 +45,8 @@
 # include "jpoll.h"
 #endif
 
+#include <oop.h>
+
 /* Workaround for some version of FreeBSD. */
 #ifdef POLLRDNORM
 # define MY_POLLIN (POLLIN | POLLRDNORM)
@@ -67,323 +70,14 @@
 
 #include "io.c.x"
 
-/* Calls trigged by a signal handler. */
-/* GABA:
-   (class
-     (name lsh_signal_handler)
-     (super resource)
-     (vars
-       (next object lsh_signal_handler)
-       (flag . "volatile sig_atomic_t *")
-       (action object lsh_callback)))
-*/
+/* Glue to liboop */
 
-/* Scheduled calls. FIXME: For now, no scheduling in time. */
-/* GABA:
-   (class
-     (name lsh_callout)
-     (super resource)
-     (vars
-       (next object lsh_callout)
-       (action object lsh_callback)))
-       ;; (when . time_t)
-*/
+/* Because of signal handlers, there can be only one oop object. */
+static oop_source_sys *global_oop_sys = NULL;
+static oop_source *source = NULL;
 
-/* GABA:
-   (class
-     (name io_backend)
-     (super resource)
-     (vars
-       ; Linked list of fds. 
-       (files object lsh_fd)
-
-       ; Stack of closed files
-       ;; (closed object lsh_fd)
-
-       ; Flags
-       (signals object lsh_signal_handler)
-       
-       ; Callouts
-       (callouts object lsh_callout)))
-*/
-
-
-/* Backend loop */
-
-/* If there's nothing to do for this amount of time (ms), do
- * spontaneous gc. */
-
-#define IDLE_TIME 100
-
-int io_iter(struct io_backend *b)
-{
-  unsigned long nfds; /* FIXME: Should be nfds_t if that type is defined */
-  struct pollfd *fds;
-  struct lsh_fd **active_fds;
-  
-  /* FIXME: Real scheduling of callouts not implemented */
-  /* int timeout; */
-  int res;
-
-  /* Check all flags */
-  if (b->signals)
-    {
-      struct lsh_signal_handler *f;
-      struct lsh_signal_handler **f_p;
-      for (f_p = &b->signals; (f = *f_p); )
-	{
-	  if (!f->super.alive)
-	    *f_p = f->next;
-	  else
-	    {
-	      if (*f->flag)
-		{
-		  *f->flag = 0;
-		  LSH_CALLBACK(f->action);
-		}
-	      f_p = &f->next;
-	    }
-	}
-    }
-  
-  /* Invoke all callouts. Clear the list first; if any callout
-   * installs another one, that will not be invoked until the next
-   * iteration. */
-  if (b->callouts)
-    {
-      struct lsh_callout *p;
-      
-      for (p = b->callouts, b->callouts = NULL;
-	   p; p = p->next)
-	if (p->super.alive)
-	  {
-	    LSH_CALLBACK(p->action);
-	    p->super.alive = 0;
-	  }
-    }
-
-  /* Prepare fds. */
-  {
-    struct lsh_fd *fd;
-
-    for (fd = b->files; fd; fd = fd->next)
-      /* NOTE: The prepare callback is allowed to close files. */
-      if (fd->super.alive && fd->prepare)
-	FD_PREPARE(fd);
-  }
-
-  /* This phase unlinks any closed fd:s, and also counts how many
-   * fd:s there are. */
-  {
-    struct lsh_fd *fd;
-    struct lsh_fd **fd_p;
-    nfds = 0;
-    
-    for(fd_p = &b->files; (fd = *fd_p); )
-      {
-	if (!fd->super.alive)
-	  /* Unlink this fd */
-	  *fd_p = fd->next;
-	    
-	else
-	  {
-	    if (fd->want_read || fd->want_write)
-	      nfds++;
-
-	    fd_p = &fd->next;
-	  }
-      }
-  }
-    
-  if (!nfds)
-    /* Nothing more to do.
-     *
-     * NOTE: There might be some callouts left, but we won't wait for them. */
-    return 0;
-
-  fds = alloca(sizeof(struct pollfd) * nfds);
-  active_fds = alloca(sizeof(struct lsh_fd *) *nfds);
-  
-  /* Fill out fds-array */
-  {
-    struct lsh_fd *fd;
-    unsigned long i;
-    int all_events = 0;
-    
-    for (fd = b->files, i = 0; fd; fd = fd->next)
-      {
-	assert(fd->super.alive);
-	
-	if (fd->want_read || fd->want_write)
-	  {
-	    assert(i < nfds);
-
-	    active_fds[i] = fd;
-
-	    fds[i].fd = fd->fd;
-	    fds[i].events = 0;
-	    
-	    if (fd->want_read)
-	      fds[i].events |= MY_POLLIN;
-
-	    if (fd->want_write)
-	      fds[i].events |= POLLOUT;
-
-	    all_events |= fds[i].events;
-	    i++;
-	  }
-      }
-    assert(i == nfds);
-    assert(all_events);
-  }
-
-  res = poll(fds, nfds, IDLE_TIME);
-
-  if (!res)
-    {
-      gc_maybe(&b->super.super, 0);
-      res = poll(fds, nfds, -1);
-    }
-  else
-    gc_maybe(&b->super.super, 1);
-  
-  if (!res)
-    {
-      /* Callouts are not implemented */
-      fatal("Unexpected timeout\n");
-    }
-  if (res < 0)
-    switch(errno)
-      {
-      case EAGAIN:
-      case EINTR:
-	return 1;
-      default:
-	fatal("io_iter: poll failed: %z", STRERROR(errno));
-      }
-  
-  {
-    /* Do io. Note that the callback functions may add new fds to the
-     * head of the list, or clear the alive flag on any fd. But this
-     * is less of a problem now, as we use the active_fd array.*/
-
-    /* struct lsh_fd *fd; */
-    unsigned long i;
-    
-    for(i=0; i<nfds; i++)
-      {
-	struct lsh_fd *fd = active_fds[i];
-	assert(i<nfds);
-
-	debug("io.c: poll for fd %i: events = 0x%xi, revents = 0x%xi.\n",
- 	      fds[i].fd, fds[i].events, fds[i].revents);
-	
-	if (!fd->super.alive)
-	  continue;
-
-	/* On systems without poll, we use jpoll.c to emulate some
-	 * of poll, but we lack POLLNVAL, POLLPRI and POLLHUP. */
-#ifdef POLLNVAL
-	if (fds[i].revents & POLLNVAL)
-	  {
-	    werror("io.c: poll request on fd %i, for events of type %xi\n"
-		   "      return POLLNVAL, revents = %xi\n",
-		   fds[i].fd, fds[i].events, fds[i].revents);
-	    close_fd(fd);
-	    continue;
-	  }
-#endif /* POLLNVAL */
-
-#ifdef POLLHUP
-	/* NOTE: The behaviour of poll at EOF varies quite a lot
-	 * between systems.
-	 *
-	 * According to Solaris' man page, POLLHUP is mutually
-	 * exclusive with POLLOUT, but orthogonal to POLLIN.
-	 *
-	 * However, on my system (sparc-linux) POLLHUP is set when we
-	 * get EOF on an fd we are reading.
-	 *
-	 * I.e. on some systems, EOF is indicated by poll setting
-	 * POLLIN and read returning 0 (in particular, this happens
-	 * if the poll-by-select-code in jpoll.c is used), while on
-	 * other systems, poll sets POLLHUP and subsequent read
-	 * calls will return -1, not 0.
-	 *
-	 * But to complicate things some more, we can (also on Linux)
-	 * get both POLLHUP and POLLIN set. In that case, we do an
-	 * ordinary read.
-	 *
-	 * We set the hanged_up flag before calling FD_READ, which
-	 * tells the io_callback that it should avoid calling read. */
-
-	if (fds[i].revents & POLLHUP)
-	  {
-	    if (fd->want_write)
-	      /* Will raise an i/o error */
-	      FD_WRITE(fd);
-	    else if (fd->want_read)
-	      {
-		if (!fd->super.alive)
-		  continue;
-
-		/* If reading is not possible, treat this as EOF. */ 
-		if (!(fds[i].revents & MY_POLLIN))
-		  fd->hanged_up = 1;
-
-		FD_READ(fd);
-	      }
-	    else
-	      {
-		werror("io.c: poll said POLLHUP on an inactive fd.\n");
-		close_fd(fd);
-	      }
-	    continue;
-	  }
-#endif /* POLLHUP */
-
-#ifdef POLLERR
-	if (fds[i].revents & POLLERR)
-	  {
-	    werror("io.c: POLLERR. Hanging up.\n");
-
-	    /* FIXME: Should we raise any exception here? */
-
-	    close_fd(fd); 
-
-	    continue;
-	  }
-#endif /* POLLERR */
-	
-#ifdef POLLPRI
-	if (fds[i].revents & POLLPRI)
-	  {
-	    werror("io.c: Peer is trying to send Out of Band data. Hanging up.\n");
-
-	    /* FIXME: Should we raise any exception here? */
-
-	    close_fd(fd); 
-
-	    continue;
-	  }
-#endif /* POLLPRI */
-	if (fds[i].revents & POLLOUT)
-	  FD_WRITE(fd);
-
-	if (!fd->super.alive)
-	  continue;
-
-	if (fds[i].revents & MY_POLLIN)
-	  FD_READ(fd);
-      }
-    assert(i == nfds);
-  }
-
-  return 1;
-}
-  
 void
-io_run(struct io_backend *b)
+io_init(void)
 {
   struct sigaction pipe;
   memset(&pipe, 0, sizeof(pipe));
@@ -394,87 +88,275 @@ io_run(struct io_backend *b)
   
   if (sigaction(SIGPIPE, &pipe, NULL) < 0)
     fatal("Failed to ignore SIGPIPE.\n");
-  
-  while(io_iter(b))
-    ;
-}
 
-static void
-do_kill_io_backend(struct resource *s)
-{
-  CAST(io_backend, backend, s);
+  assert(!global_oop_sys);
+  global_oop_sys = oop_sys_new();
+  if (!global_oop_sys)
+    fatal("Failed to initialize liboop.\n");
 
-  if (backend->super.alive)
-    {
-      struct lsh_fd *fd;
-      struct lsh_signal_handler *signal;
-  
-      for (fd = backend->files, backend->files = NULL;
-	   fd; fd = fd->next)
-	close_fd(fd);
-
-      /* Check that no callback has opened new files. */
-      assert(!backend->files);
-
-      for (signal = backend->signals, backend->signals = NULL;
-	   signal; signal = signal->next)
-	signal->super.alive = 0;
-
-      backend->super.alive = 0;
-    }
-}
-
-struct io_backend *
-make_io_backend(void)
-{
-  NEW(io_backend, b);
-
-  init_resource(&b->super, do_kill_io_backend);
-  
-  b->files = NULL;
-  b->signals = NULL;
-  b->callouts = NULL;
-
-  return b;
+  source = oop_sys_source(global_oop_sys);
 }
 
 void
-io_final(struct io_backend *b)
+io_final(void)
 {
-  KILL_RESOURCE(&b->super);
+  gc_final();
+  
+  /* There mustn't be any outstanding callbacks left. */
+  oop_sys_delete(global_oop_sys);
+  global_oop_sys = NULL;
+  source = NULL;
+}
+
+void
+io_run(void)
+{
+  void *res = oop_sys_run(global_oop_sys);
+
+  /* FIXME: OOP_ERROR is not defined in liboop-0.7. Upgrade, and then
+   * enable this check. */
+#if 0
+  if (res == OOP_ERROR)
+    werror("oop_sys_run (errno = %i):  %z\n",
+	   errno, STRERROR(errno));
+#endif
+  trace("io_run: Exiting\n");
+}
+
+
+/* OOP Callbacks */
+static void *
+lsh_oop_signal_callback(oop_source *s UNUSED, int sig, void *data)
+{
+  CAST(lsh_signal_handler, self, (struct lsh_object *) data);
+
+  trace("lsh_oop_signal_callback: Signal %i, handler: %t\n",
+	sig, self->action);
+  
+  assert(sig == self->signum);
+  
+  LSH_CALLBACK(self->action);
+
+  return OOP_CONTINUE;
+}
+
+static void
+lsh_oop_register_signal(struct lsh_signal_handler *handler)
+{
+  trace("lsh_oop_register_signal: signal: %i, handler: %t\n",
+	handler->signum, handler);
+  
+  if (handler->super.alive)
+    source->on_signal(source, handler->signum, lsh_oop_signal_callback, handler);
+}
+
+static void
+lsh_oop_cancel_signal(struct lsh_signal_handler *handler)
+{
+  trace("lsh_oop_cancel_signal: signal: %i, handler: %t\n",
+	handler->signum, handler);
+
+  if (handler->super.alive)
+    source->cancel_signal(source, handler->signum, lsh_oop_signal_callback, handler);
+}
+
+static void *
+lsh_oop_fd_read_callback(oop_source *s UNUSED, int fileno, oop_event event, void *data)
+{
+  CAST(lsh_fd, fd, (struct lsh_object *) data);
+
+  assert(fileno == fd->fd);
+  assert(event == OOP_READ);
+  assert(fd->super.alive);
+
+  trace("lsh_oop_fd_read_callback: fd %i: %z\n",
+	fd->fd, fd->label);
+
+  FD_READ(fd);
+
+  return OOP_CONTINUE;
+}
+
+void
+lsh_oop_register_read_fd(struct lsh_fd *fd)
+{
+  trace("lsh_oop_register_read_fd: fd: %i, %z\n",
+	fd->fd, fd->label);
+  
+  if (fd->super.alive && !fd->want_read)
+    {
+      assert(fd->read);
+      
+      source->on_fd(source, fd->fd, OOP_READ, lsh_oop_fd_read_callback, fd);
+      fd->want_read = 1;
+    }
+}
+
+void
+lsh_oop_cancel_read_fd(struct lsh_fd *fd)
+{
+  trace("lsh_oop_cancel_read_fd: fd: %i, %z\n",
+	fd->fd, fd->label);
+  
+  if (fd->super.alive)
+    {
+      source->cancel_fd(source, fd->fd, OOP_READ);
+      fd->want_read = 0;
+    }
+}
+
+static void *
+lsh_oop_fd_write_callback(oop_source *s UNUSED, int fileno, oop_event event, void *data)
+{
+  CAST(lsh_fd, fd, (struct lsh_object *) data);
+
+  assert(fileno == fd->fd);
+  assert(event == OOP_WRITE);
+  assert(fd->super.alive);
+  
+  trace("lsh_oop_fd_write_callback: fd %i: %z\n",
+	fd->fd, fd->label);
+
+  FD_WRITE(fd);
+
+  return OOP_CONTINUE;
+}
+
+void
+lsh_oop_register_write_fd(struct lsh_fd *fd)
+{
+  trace("lsh_oop_register_write_fd: fd: %i, %z\n",
+	fd->fd, fd->label);
+  
+  if (fd->super.alive && !fd->want_write)
+    {
+      assert(fd->write);
+      
+      source->on_fd(source, fd->fd, OOP_WRITE, lsh_oop_fd_write_callback, fd);
+      fd->want_write = 1;
+    }
+}
+
+void
+lsh_oop_cancel_write_fd(struct lsh_fd *fd)
+{
+  trace("lsh_oop_cancel_write_fd: fd: %i, %z\n",
+	fd->fd, fd->label);
+
+  if (fd->super.alive)
+    {
+      source->cancel_fd(source, fd->fd, OOP_WRITE);
+      fd->want_write = 0;
+    }
+}
+
+static void *
+lsh_oop_time_callback(oop_source *source UNUSED,
+                      struct timeval time UNUSED, void *data)
+{
+  CAST(lsh_callout, callout, (struct lsh_object *) data);
+
+  assert(callout->super.alive);
+
+  trace("lsh_oop_time_callback: action: %t\n",
+        callout->action);
+  
+  callout->super.alive = 0;
+
+  LSH_CALLBACK(callout->action);
+
+  return OOP_CONTINUE;
+}
+
+static void
+lsh_oop_register_callout(struct lsh_callout *callout)
+{
+  if (callout->super.alive)
+    source->on_time(source, OOP_TIME_NOW, lsh_oop_time_callback, callout);
+}
+
+static void
+lsh_oop_cancel_callout(struct lsh_callout *callout)
+{
+  if (callout->super.alive)
+    source->cancel_time(source, OOP_TIME_NOW, lsh_oop_time_callback, callout);
+}
+
+
+/* Calls trigged by a signal handler. */
+/* GABA:
+   (class
+     (name lsh_signal_handler)
+     (super resource)
+     (vars
+       (signum . int)
+       (action object lsh_callback)))
+*/
+
+/* Scheduled calls. FIXME: For now, no scheduling in time. */
+/* GABA:
+   (class
+     (name lsh_callout)
+     (super resource)
+     (vars
+       (action object lsh_callback)))
+*/
+
+
+static void
+do_kill_signal_handler(struct resource *s)
+{
+  CAST(lsh_signal_handler, self, s);
+
+  if (self->super.alive)
+    {
+      lsh_oop_cancel_signal(self);
+      self->super.alive = 0;
+    }
 }
 
 struct resource *
-io_signal_handler(struct io_backend *b,
-		  volatile sig_atomic_t *flag,
+io_signal_handler(int signum,
 		  struct lsh_callback *action)
 {
   NEW(lsh_signal_handler, handler);
-  init_resource(&handler->super, NULL);
 
-  handler->next = b->signals;
-  handler->flag = flag;
+  init_resource(&handler->super, do_kill_signal_handler);
+
+  handler->signum = signum;
   handler->action = action;
 
-  b->signals = handler;
+  lsh_oop_register_signal(handler);
+  gc_global(&handler->super);
   
   return &handler->super;
 }
 
+static void
+do_kill_callout(struct resource *s)
+{
+  CAST(lsh_callout, self, s);
+
+  if (self->super.alive)
+    {
+      lsh_oop_cancel_callout(self);
+      self->super.alive = 0;
+    }
+}
+
 /* Delays not implemented. */
 struct resource *
-io_callout(struct io_backend *b,
-	   UINT32 delay UNUSED,
-	   struct lsh_callback *action)
+io_callout(struct lsh_callback *action)
 {
-  NEW(lsh_callout, callout);
-  init_resource(&callout->super, NULL);
+  NEW(lsh_callout, self);
+  init_resource(&self->super, do_kill_callout);
 
-  callout->next = b->callouts;
-  callout->action = action;
-  b->callouts = callout;
+  self->action = action;
 
-  return &callout->super;
+  lsh_oop_register_callout(self);
+  
+  gc_global(&self->super);
+  return &self->super;
 }
 
 /* Read-related callbacks */
@@ -489,9 +371,13 @@ do_buffered_read(struct io_callback *s,
 
   assert(fd->want_read);   
 
+#if 0
   /* If hanged_up is set, pretend that read returned 0 */
   res = fd->hanged_up ? 0 : read(fd->fd, buffer, self->buffer_size);
+#endif
 
+  res = read(fd->fd, buffer, self->buffer_size);
+  
   if (res < 0)
     switch(errno)
       {
@@ -554,7 +440,7 @@ do_buffered_read(struct io_callback *s,
 	    {
 	      werror("do_buffered_read: Handler disappeared! Ignoring %i bytes\n",
 		     left);
-	      fd->want_read = 0;
+	      lsh_oop_cancel_read_fd(fd);
 	      return;
 	    }
 	}
@@ -600,16 +486,10 @@ do_consuming_read(struct io_callback *c,
   UINT32 wanted = READ_QUERY(self);
 
   assert(fd->want_read);
-
-  if (fd->hanged_up)
-    {
-      /* If hanged_up is set, pretend that read returned 0 */
-      goto eof;
-    }
   
   if (!wanted)
     {
-      fd->want_read = 0;
+      lsh_oop_cancel_read_fd(fd);
     }
   else
     {
@@ -630,6 +510,7 @@ do_consuming_read(struct io_callback *c,
 	       * EPIPE, but it happens occasionally under linux. Perhaps
 	       * we should treat it as EOF instead? */
 	      werror("io.c: read_consume: Unexpected EPIPE.\n");
+	      /* Fall through */
 	    default:
 	      EXCEPTION_RAISE(fd->e, 
 			      make_io_exception(EXC_IO_READ,
@@ -646,7 +527,7 @@ do_consuming_read(struct io_callback *c,
       else
 	{
 	  lsh_string_free(s);
-	eof:
+
 	  /* Close the fd, unless it has a write callback. */
 	  A_WRITE(self->consumer, NULL);
 	  close_fd_read(fd);
@@ -670,62 +551,56 @@ do_write_callback(struct io_callback *s UNUSED,
 		  struct lsh_fd *fd)
 {
   /* CAST(io_write_callback, self, s); */
-  UINT32 size;
-  int res;
+  assert(fd->super.alive);
   
-  size = MIN(fd->write_buffer->end - fd->write_buffer->start,
-	     fd->write_buffer->block_size);
-  assert(size);
-  
-  res = write(fd->fd,
-	      fd->write_buffer->buffer + fd->write_buffer->start,
-	      size);
-  if (!res)
-    fatal("Closed?");
-  if (res < 0)
-    switch(errno)
-      {
-      case EINTR:
-      case EAGAIN:
-	break;
-      case EPIPE:
-	debug("io.c: Broken pipe.\n");
-	
-	/* Fall through */
-      default:
-	werror("io.c: write failed, %z\n", STRERROR(errno));
-	EXCEPTION_RAISE(fd->e,
-			make_io_exception(EXC_IO_WRITE, fd, errno, NULL));
+  if (!write_buffer_pre_write(fd->write_buffer))
+    {
+      /* Buffer is empty */
+      if (fd->write_buffer->closed)
 	close_fd(fd);
-	
-	break;
-      }
+      else
+	lsh_oop_cancel_write_fd(fd);
+    }
   else
-    write_buffer_consume(fd->write_buffer, res);
-}  
+    {
+      UINT32 size;
+      int res;
+
+      size = MIN(fd->write_buffer->end - fd->write_buffer->start,
+		 fd->write_buffer->block_size);
+      assert(size);
+  
+      res = write(fd->fd,
+		  fd->write_buffer->buffer + fd->write_buffer->start,
+		  size);
+      if (!res)
+	fatal("Closed?");
+      if (res < 0)
+	switch(errno)
+	  {
+	  case EINTR:
+	  case EAGAIN:
+	    break;
+	  case EPIPE:
+	    debug("io.c: Broken pipe.\n");
+	
+	    /* Fall through */
+	  default:
+	    werror("io.c: write failed, %z\n", STRERROR(errno));
+	    EXCEPTION_RAISE(fd->e,
+			    make_io_exception(EXC_IO_WRITE, fd, errno, NULL));
+	    close_fd(fd);
+	
+	    break;
+	  }
+      else
+	write_buffer_consume(fd->write_buffer, res);
+    }
+}
 
 static struct io_callback io_write_callback =
 { STATIC_HEADER, do_write_callback };
 
-static void
-do_write_prepare(struct lsh_fd *fd)
-{
-  assert(fd->write_buffer);
-
-  if (! (fd->want_write = write_buffer_pre_write(fd->write_buffer))
-      && fd->write_buffer->closed)
-    close_fd(fd);
-}
-
-static void
-do_write_close(struct lsh_fd *fd)
-{
-  /* CAST(io_write_callback, self, s); */
-
-  assert(fd->write_buffer);
-
-  write_buffer_close(fd->write_buffer);
-}
 
 struct listen_value *
 make_listen_value(struct lsh_fd *fd,
@@ -747,7 +622,6 @@ make_listen_value(struct lsh_fd *fd,
      (name io_listen_callback)
      (super io_callback)
      (vars
-       (backend object io_backend)
        (c object command_continuation)
        (e object exception_handler)))
 */
@@ -776,20 +650,17 @@ do_listen_callback(struct io_callback *s,
     }
   trace("io.c: accept on fd %i\n", conn);
   COMMAND_RETURN(self->c,
-		 make_listen_value(make_lsh_fd(self->backend,
-					       conn, "accepted socket", self->e),
+		 make_listen_value(make_lsh_fd(conn, "accepted socket", self->e),
 				   sockaddr2info(addr_len,
 						 (struct sockaddr *) &peer)));
 }
 
 struct io_callback *
-make_listen_callback(struct io_backend *backend,
-		     struct command_continuation *c,
+make_listen_callback(struct command_continuation *c,
 		     struct exception_handler *e)
 {
   NEW(io_listen_callback, self);
   self->super.f = do_listen_callback;
-  self->backend = backend;
   self->c = c;
   self->e = e;
   
@@ -819,15 +690,16 @@ do_connect_callback(struct io_callback *s,
 		  (char *) &socket_error, &len) < 0)
       || socket_error)
     {
-      debug("io.c: connect_callback: Connect failed.\n");
+      trace("io.c: connect_callback: Connect on fd %i failed.\n", fd->fd);
       EXCEPTION_RAISE(fd->e,
 		      make_io_exception(EXC_IO_CONNECT, fd, 0, "connect failed."));
       close_fd(fd);
     }
   else
     {
+      trace("io.c: connect_callback: fd %i connected.\n", fd->fd);
       fd->write = NULL;
-      fd->want_write = 0;
+      lsh_oop_cancel_write_fd(fd);
       fd->label = "connected socket";
       COMMAND_RETURN(self->c, fd);
     }
@@ -877,35 +749,6 @@ do_exc_io_handler(struct exception_handler *self,
   return;
 }
 
-/* Initializes a file structure, and adds it to the backend's list. */
-static void
-init_file(struct io_backend *b, struct lsh_fd *f, int fd,
-	  const char *label,
-	  struct exception_handler *e)
-{
-  init_resource(&f->super, do_kill_fd);
-
-  f->fd = fd;
-  f->label = label;
-  
-  f->e = make_exception_handler(do_exc_io_handler, e, HANDLER_CONTEXT);
-  
-  f->close_callback = NULL;
-
-  f->prepare = NULL;
-
-  f->hanged_up = 0;
-  
-  f->want_read = 0;
-  f->read = NULL;
-
-  f->want_write = 0;
-  f->write = NULL;
-  f->write_close = NULL;
-
-  f->next = b->files;
-  b->files = f;
-}
 
 /* These functions are used by werror and friends */
 
@@ -1010,6 +853,43 @@ read_raw(int fd, UINT32 length, UINT8 *data)
   return NULL;
 }
 
+struct lsh_string *
+io_read_file_raw(int fd, UINT32 guess)
+{
+  struct string_buffer buffer;
+  string_buffer_init(&buffer, guess);
+
+  for (;;)
+    {
+      int res;
+      
+      if (!buffer.left)
+        /* Roughly double the size of the buffer */
+        string_buffer_grow(&buffer,
+                           buffer.partial->length + buffer.total + 100);
+      
+      res = read(fd, buffer.current, buffer.left);
+      
+      if (res < 0)
+        {
+          if (errno == EINTR)
+            continue;
+          
+          string_buffer_clear(&buffer);
+          return NULL;
+        }
+      else if (!res)
+        {
+          /* EOF */
+          return string_buffer_final(&buffer, buffer.left);
+        }
+      assert( (unsigned) res <= buffer.left);
+      
+      buffer.current += res;
+      buffer.left -= res;
+    }
+}
+
 /* Network utility functions */
 
 /* Converts a string port number or service name to a port number.
@@ -1198,7 +1078,7 @@ address_info2sockaddr(socklen_t *length,
 
     if (err)
       {
-	debug("address_info2sockaddr: getaddrinfo failed (err = %d): %z\n",
+	debug("address_info2sockaddr: getaddrinfo failed (err = %i): %z\n",
 	      err, gai_strerror(err));
 	return NULL;
       }
@@ -1331,22 +1211,35 @@ void io_init_fd(int fd)
 }
 
 struct lsh_fd *
-make_lsh_fd(struct io_backend *b,
-	    int fd, const char *label,
+make_lsh_fd(int fd, const char *label,
 	    struct exception_handler *e)
 {
-  NEW(lsh_fd, f);
+  NEW(lsh_fd, self);
 
   io_init_fd(fd);
-  init_file(b, f, fd, label, e);
 
-  return f;
+  init_resource(&self->super, do_kill_fd);
+
+  self->fd = fd;
+  self->label = label;
+  
+  self->e = make_exception_handler(do_exc_io_handler, e, HANDLER_CONTEXT);
+  
+  self->close_callback = NULL;
+
+  self->want_read = 0;
+  self->read = NULL;
+
+  self->want_write = 0;
+  self->write = NULL;
+
+  gc_global(&self->super);
+  return self;
 }
 
 /* Some code is taken from Thomas Bellman's tcputils. */
 struct lsh_fd *
-io_connect(struct io_backend *b,
-	   struct sockaddr *remote,
+io_connect(struct sockaddr *remote,
 	   socklen_t remote_length,
 	   struct command_continuation *c,
 	   struct exception_handler *e)
@@ -1380,17 +1273,16 @@ io_connect(struct io_backend *b,
       return NULL;
     }
 
-  fd = make_lsh_fd(b, s, "connecting socket", e);
+  fd = make_lsh_fd(s, "connecting socket", e);
   
-  fd->want_write = 1;
   fd->write = make_connect_callback(c);
+  lsh_oop_register_write_fd(fd);
     
   return fd;
 }
 
 struct lsh_fd *
-io_listen(struct io_backend *b,
-	  struct sockaddr *local,
+io_listen(struct sockaddr *local,
 	  socklen_t length,
 	  struct io_callback *callback,
 	  struct exception_handler *e)
@@ -1422,10 +1314,10 @@ io_listen(struct io_backend *b,
       return NULL;
     }
 
-  fd = make_lsh_fd(b, s, "listening socket", e);
+  fd = make_lsh_fd(s, "listening socket", e);
 
-  fd->want_read = 1;
   fd->read = callback;
+  lsh_oop_register_read_fd(fd);
 
   return fd;
 }
@@ -1546,8 +1438,7 @@ safe_pushd(const char *directory,
 
 
 struct lsh_fd *
-io_listen_local(struct io_backend *b,
-		struct local_info *info,
+io_listen_local(struct local_info *info,
 		struct io_callback *callback,
 		struct exception_handler *e)
 {
@@ -1600,7 +1491,7 @@ io_listen_local(struct io_backend *b,
   old_umask = umask(0077);
 
   /* Bind and listen */
-  fd = io_listen(b, (struct sockaddr *) local, local_length, callback, e);
+  fd = io_listen( (struct sockaddr *) local, local_length, callback, e);
   
   /* Ok, now we restore umask and cwd */
   umask(old_umask);
@@ -1612,8 +1503,7 @@ io_listen_local(struct io_backend *b,
 
 /* Requires DIRECTORY and NAME to be NUL-terminated */
 struct lsh_fd *
-io_connect_local(struct io_backend *b,
-		 struct local_info *info,
+io_connect_local(struct local_info *info,
 		 struct command_continuation *c,
 		 struct exception_handler *e)
 {
@@ -1645,7 +1535,7 @@ io_connect_local(struct io_backend *b,
   if (old_cd < 0)
     return NULL;
   
-  fd = io_connect(b, (struct sockaddr *) addr, addr_length, c, e);
+  fd = io_connect( (struct sockaddr *) addr, addr_length, c, e);
 
   safe_popd(old_cd, cdir);
 
@@ -1665,15 +1555,13 @@ io_read_write(struct lsh_fd *fd,
   
   /* Reading */
   fd->read = read;
-  fd->want_read = !!read;
+  if (read)
+    lsh_oop_register_read_fd(fd);
   
   /* Writing */
-  fd->write_buffer = make_write_buffer(block_size);
+  fd->write_buffer = make_write_buffer(fd, block_size);
   fd->write = &io_write_callback;
 
-  fd->prepare = do_write_prepare;
-  fd->write_close = do_write_close;
-  
   /* Closing */
   fd->close_callback = close_callback;
 
@@ -1688,8 +1576,9 @@ io_read(struct lsh_fd *fd,
   trace("io.c: Preparing fd %i for reading\n", fd->fd);
   
   /* Reading */
-  fd->want_read = !!read;
   fd->read = read;
+  if (fd->read)
+    lsh_oop_register_read_fd(fd);
   
   fd->close_callback = close_callback;
 
@@ -1704,20 +1593,16 @@ io_write(struct lsh_fd *fd,
   trace("io.c: Preparing fd %i for writing\n", fd->fd);
   
   /* Writing */
-  fd->write_buffer = make_write_buffer(block_size);
+  fd->write_buffer = make_write_buffer(fd, block_size);
   fd->write = &io_write_callback;
 
-  fd->prepare = do_write_prepare;
-  fd->write_close = do_write_close;
-
   fd->close_callback = close_callback;
 
   return fd;
 }
 
 struct lsh_fd *
-io_write_file(struct io_backend *backend,
-	      const char *fname, int flags, int mode,
+io_write_file(const char *fname, int flags, int mode,
 	      UINT32 block_size,
 	      struct lsh_callback *c,
 	      struct exception_handler *e)
@@ -1726,29 +1611,32 @@ io_write_file(struct io_backend *backend,
   if (fd < 0)
     return NULL;
 
-  return io_write(make_lsh_fd(backend, fd, "write-only file", e),
+  return io_write(make_lsh_fd(fd, "write-only file", e),
 		  block_size, c);
 }
 
 struct lsh_fd *
-io_read_file(struct io_backend *backend,
-	     const char *fname, 
+io_read_file(const char *fname, 
 	     struct exception_handler *e)
 {
   int fd = open(fname, O_RDONLY);
   if (fd < 0)
     return NULL;
 
-  return make_lsh_fd(backend, fd, "read-only file", e);
+  return make_lsh_fd(fd, "read-only file", e);
 }
 
-void close_fd(struct lsh_fd *fd)
+void
+close_fd(struct lsh_fd *fd)
 {
   trace("io.c: Closing fd %i: %z.\n",
 	fd->fd, fd->label);
 
   if (fd->super.alive)
     {
+      lsh_oop_cancel_read_fd(fd);
+      lsh_oop_cancel_write_fd(fd);
+
       fd->super.alive = 0;
   
       if (fd->fd < 0)
@@ -1758,12 +1646,12 @@ void close_fd(struct lsh_fd *fd)
   
       /* Used by write fd:s to make sure that writing to its
        * buffer fails. */
-      if (fd->write_close)
-	FD_WRITE_CLOSE(fd);
+      if (fd->write_buffer)
+	fd->write_buffer->closed = 1;
   
       if (fd->close_callback)
 	LSH_CALLBACK(fd->close_callback);
-  
+      
       if (close(fd->fd) < 0)
 	{
 	  werror("io.c: close failed, (errno = %i): %z\n",
@@ -1777,29 +1665,36 @@ void close_fd(struct lsh_fd *fd)
     werror("Closed already.\n");
 }
 
-void close_fd_nicely(struct lsh_fd *fd)
+void
+close_fd_nicely(struct lsh_fd *fd)
 {
   /* Don't attempt to read any further. */
 
   trace("io.c: close_fd_nicely called on fd %i: %z\n",
 	fd->fd, fd->label);
   
-  fd->want_read = 0;
   fd->read = NULL;
+
+  lsh_oop_cancel_read_fd(fd);
   
-  if (fd->write_close)
-    /* Mark the write_buffer as closed */
-    FD_WRITE_CLOSE(fd);
-  else
-    /* There's no data buffered for write. */
-    close_fd(fd);
+  if (fd->write_buffer)
+    {
+      /* Mark the write_buffer as closed */
+      fd->write_buffer->closed = 1;
+      if (!fd->write_buffer->empty)
+	return;
+    }
+
+  /* There's no data buffered for write. */
+  close_fd(fd);
 }
 
 /* Stop reading, but if the fd has a write callback, keep it open. */
 void close_fd_read(struct lsh_fd *fd)
 {
-  fd->want_read = 0;
   fd->read = NULL;
+
+  lsh_oop_cancel_read_fd(fd);
   
   if (!fd->write)
     /* We won't be writing anything on this fd, so close it. */
@@ -1832,11 +1727,11 @@ do_exc_finish_read_handler(struct exception_handler *s,
       close_fd(self->fd);
       break;
     case EXC_PAUSE_READ:
-      self->fd->want_read = 0;
+      lsh_oop_cancel_read_fd(self->fd);
       break;
     case EXC_PAUSE_START_READ:
       if (self->fd->read)
-	self->fd->want_read = 1;
+	lsh_oop_register_read_fd(self->fd);
       break;
     default:
       EXCEPTION_RAISE(self->super.parent, e);
@@ -1896,7 +1791,7 @@ lsh_make_pipe(int *fds)
       werror("socketpair failed: %z\n", STRERROR(errno));
       return 0;
     }
-  debug("Created socket pair. Using fd:s %i <-- %i\n", fds[0], fds[1]);
+  trace("Created socket pair. Using fd:s %i <-- %i\n", fds[0], fds[1]);
 
   if (SHUTDOWN_UNIX(fds[0], SHUT_WR_UNIX) < 0)
     {
