@@ -1,6 +1,6 @@
 /* io_commands.c
  *
- * $Id: io_commands.c,v 1.44 2001/07/06 21:13:27 nisse Exp $
+ * $Id: io_commands.c,v 1.49 2001/09/27 12:10:33 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -26,6 +26,8 @@
 
 #include "command.h"
 #include "connection.h"
+/* For lsh_get_cstring */
+#include "format.h"
 #include "io.h"
 #include "werror.h"
 #include "xalloc.h"
@@ -45,30 +47,21 @@
 #include "io_commands.c.x"
 
 /* Used only by lsh-writekey */
-/* GABA:
-   (class
-     (name backend_command)
-     (super command)
-     (vars
-       (backend object io_backend)))
-*/
 
-/* (write file_info backend)
+/* (write file_info)
  *
  * Opens a file for write, and returns the corresponding write_buffer.
  * */
-   
-static void
-do_io_write_file(struct command *s,
-		 struct lsh_object *a,
-		 struct command_continuation *c,
-		 struct exception_handler *e)
+
+DEFINE_COMMAND(io_write_file_command)
+     (struct command *s UNUSED,
+      struct lsh_object *a,
+      struct command_continuation *c,
+      struct exception_handler *e)
 {
-  CAST(backend_command, self, s);
   CAST(io_write_file_info, info, a);
 
-  struct lsh_fd *fd = io_write_file(self->backend,
-				    info->name,
+  struct lsh_fd *fd = io_write_file(lsh_get_cstring(info->name),
 				    info->flags,
 				    info->mode,
 				    info->block_size,
@@ -80,23 +73,8 @@ do_io_write_file(struct command *s,
     EXCEPTION_RAISE(e, make_io_exception(EXC_IO_OPEN_WRITE, NULL, errno, NULL));
 }
 
-DEFINE_COMMAND(io_write_file_command)
-     (struct command *s UNUSED,
-      struct lsh_object *a,
-      struct command_continuation *c,
-      struct exception_handler *e UNUSED)
-{
-  CAST(io_backend, backend, a);
-
-  NEW(backend_command, self);
-  self->super.call = do_io_write_file;
-  self->backend = backend;
-
-  COMMAND_RETURN(c, self);
-}
-
 struct io_write_file_info *
-make_io_write_file_info(const char *name, int flags, int mode, UINT32 block_size)
+make_io_write_file_info(struct lsh_string *name, int flags, int mode, UINT32 block_size)
 {
   NEW(io_write_file_info, self);
   self->name = name;
@@ -107,22 +85,6 @@ make_io_write_file_info(const char *name, int flags, int mode, UINT32 block_size
   return self;
 }
 
-/* FIXME: Used only by lsh-writekey. Delete? */
-void do_io_read_fd(struct command *s,
-		   struct lsh_object *a,
-		   struct command_continuation *c,
-		   struct exception_handler *e)
-{
-  CAST(io_read_fd, self, s);
-  CAST(io_backend, backend, a);
-
-  COMMAND_RETURN(c, make_lsh_fd(backend,
-				self->fd, "io_read_stdin", e));
-}
-
-struct io_read_fd io_read_stdin
-= STATIC_IO_READ_FD(STDIN_FILENO);
-
 
 static struct exception resolve_exception =
 STATIC_EXCEPTION(EXC_RESOLVE, "address could not be resolved");
@@ -130,8 +92,7 @@ STATIC_EXCEPTION(EXC_RESOLVE, "address could not be resolved");
 /* Used by do_listen_callback and any other listen variants. Currently
  * doesn't perform any dns lookups. */
 static void
-do_listen(struct io_backend *backend,
-	  struct address_info *a,
+do_listen(struct address_info *a,
 	  /* Continuation if listen succeeds. */
 	  struct command_continuation *listen_c,
 	  /* Continuation if accept succeeds. */
@@ -150,9 +111,8 @@ do_listen(struct io_backend *backend,
       return;
     }
 
-  fd = io_listen(backend,
-		 addr, addr_length,
-		 make_listen_callback(backend, accept_c, e),
+  fd = io_listen(addr, addr_length,
+		 make_listen_callback(accept_c, e),
 		 e);
   lsh_space_free(addr);
   
@@ -163,25 +123,24 @@ do_listen(struct io_backend *backend,
     COMMAND_RETURN(listen_c, fd);
 }
 
-/* A listen function taking three arguments:
- * (listen callback backend port).
+/* A listen function taking two arguments:
+ * (listen callback port).
  *
  * Suitable for handling forwarding requests. NOTE: The calling
  * function has to do all remembering of the fd:s. */
 
-DEFINE_COMMAND3(listen_with_callback)
-     (struct lsh_object *a1,
+DEFINE_COMMAND2(listen_with_callback)
+     (struct command_2 *s UNUSED,
+      struct lsh_object *a1,
       struct lsh_object *a2,
-      struct lsh_object *a3,
       struct command_continuation *c,
       struct exception_handler *e)
 {
   CAST_SUBTYPE(command, callback, a1);
-  CAST(io_backend, backend, a2);
-  CAST(address_info, address, a3);
+  CAST(address_info, address, a2);
 
   /* No dns lookups */
-  do_listen(backend, address,
+  do_listen(address,
 	    c,
 	    make_apply(callback,
 		       &discard_continuation, e), e);
@@ -220,8 +179,7 @@ make_connect_continuation(struct address_info *target,
 }
      
 static void
-do_connect(struct io_backend *backend,
-	   struct address_info *a,
+do_connect(struct address_info *a,
 	   struct resource_list *resources,
 	   struct command_continuation *c,
 	   struct exception_handler *e)
@@ -243,7 +201,7 @@ do_connect(struct io_backend *backend,
 
   /* If the name is canonicalized in any way, we should pass the
    * canonical name to make_connect_continuation .*/
-  fd = io_connect(backend, addr, addr_length, 
+  fd = io_connect(addr, addr_length, 
 		  make_connect_continuation(a, c), e);
   lsh_space_free(addr);
 
@@ -262,14 +220,13 @@ do_connect(struct io_backend *backend,
 /* Connect variant, taking a connection object as argument (used for
  * rememembering the connected fd).
  *
- * (connect backend port connection) -> fd */
+ * (connect port connection) -> fd */
 
 /* GABA:
    (class
      (name connect_port)
      (super command)
      (vars
-       (backend object io_backend)
        (target object address_info)))
 */
 
@@ -282,93 +239,45 @@ do_connect_port(struct command *s,
   CAST(connect_port, self, s);
   CAST(ssh_connection, connection, x);
   
-  do_connect(self->backend, self->target, connection->resources, c, e);
+  do_connect(self->target, connection->resources, c, e);
 }
 
 
 struct command *
-make_connect_port(struct io_backend *backend,
-		  struct address_info *target)
+make_connect_port(struct address_info *target)
 {
   NEW(connect_port, self);
   self->super.call = do_connect_port;
-  self->backend = backend;
   self->target = target;
 
   return &self->super;
 }
 
-
-/* GABA:
-   (class
-     (name simple_io_command)
-     (super command)
-     (vars
-       (backend object io_backend)
-       (resources object resource_list)))
-*/
-
-/* Simple connect function taking port only as argument. Also used for
- * listen.
- *
- * (connect address) */
-
-static void
-do_simple_connect(struct command *s,
-		  struct lsh_object *a,
-		  struct command_continuation *c,
-		  struct exception_handler *e)
+/* (connect address) */
+DEFINE_COMMAND(connect_simple_command)
+     (struct command *self UNUSED,
+      struct lsh_object *a,
+      struct command_continuation *c,
+      struct exception_handler *e)
 {
-  CAST(simple_io_command, self, s);
   CAST(address_info, address, a);
 
-  do_connect(self->backend, address, NULL, c, e);
-}
-
-struct command *
-make_simple_connect(struct io_backend *backend,
-		    struct resource_list *resources)
-{
-  NEW(simple_io_command, self);
-  self->backend = backend;
-  self->resources = resources;
-  
-  self->super.call = do_simple_connect;
-
-  return &self->super;
+  do_connect(address, NULL, c, e);
 }
 
 
-/* (connect connection port) */
-/* GABA:
-   (class
-     (name connect_connection)
-     (super command)
-     (vars
-       (backend object io_backend)))
-*/
-
-static void
-do_connect_connection(struct command *s,
-		      struct lsh_object *x,
-		      struct command_continuation *c,
-		      struct exception_handler *e UNUSED)
+/* (connect_connection connection port) */
+DEFINE_COMMAND2(connect_connection_command)
+     (struct command_2 *self UNUSED,
+      struct lsh_object *a1,
+      struct lsh_object *a2,
+      struct command_continuation *c,
+      struct exception_handler *e UNUSED)
 {
-  CAST(connect_connection, self, s);
-  CAST(ssh_connection, connection, x);
+  CAST(ssh_connection, connection, a1);
+  CAST(address_info, address, a2);
 
-  COMMAND_RETURN(c,
-		 make_simple_connect(self->backend, connection->resources));
-}
-
-struct command *
-make_connect_connection(struct io_backend *backend)
-{
-  NEW(connect_connection, self);
-  self->super.call = do_connect_connection;
-  self->backend = backend;
-
-  return &self->super;
+  do_connect(address, connection->resources, c, e);
 }
 
 
@@ -377,7 +286,6 @@ make_connect_connection(struct io_backend *backend)
      (name listen_local)
      (super command)
      (vars
-       (backend object io_backend)
        (info object local_info)))
 */
 
@@ -391,10 +299,8 @@ do_listen_local(struct command *s,
   CAST_SUBTYPE(command, callback, x);
   
   struct lsh_fd *fd
-    = io_listen_local(self->backend,
-		      self->info,
-		      make_listen_callback(self->backend,
-					   make_apply(callback,
+    = io_listen_local(self->info,
+		      make_listen_callback(make_apply(callback,
 						      &discard_continuation, e),
 					   e),
 		      e);
@@ -406,11 +312,9 @@ do_listen_local(struct command *s,
 }
 
 struct command *
-make_listen_local(struct io_backend *backend,
-		  struct local_info *info)
+make_listen_local(struct local_info *info)
 {
   NEW(listen_local, self);
-  self->backend = backend;
   self->info = info;
 
   self->super.call = do_listen_local;
@@ -418,7 +322,7 @@ make_listen_local(struct io_backend *backend,
   return &self->super;
 }
 
-/* GABA:
+/* ;; GABA:
    (class
      (name connect_local)
      (super command)
@@ -426,30 +330,30 @@ make_listen_local(struct io_backend *backend,
        (backend object io_backend)))
 */
 
+#if 0
 static void
-do_connect_local(struct command *s,
-		struct lsh_object *x,
-		struct command_continuation *c,
-		struct exception_handler *e)
+do_connect_local(struct command *self,
+                 struct lsh_object *x,
+                 struct command_continuation *c,
+                 struct exception_handler *e)
 {
-  CAST(connect_local, self, s);
   CAST(local_info, info, x);
 
-  io_connect_local(self->backend, info,
+  io_connect_local(info,
 		   make_connect_continuation(NULL, c),
 		   e);
 }
 
 struct command *
-make_connect_local(struct io_backend *backend)
+make_connect_local(void)
 {
   NEW(connect_local, self);
-  self->backend = backend;
 
-  self->super.call = do_connect_local;
+  self->call = do_connect_local;
 
-  return &self->super;
+  return self;
 }
+#endif
 
 DEFINE_COMMAND(connect_local_command)
      (struct command *s UNUSED,
@@ -457,8 +361,11 @@ DEFINE_COMMAND(connect_local_command)
       struct command_continuation *c,
       struct exception_handler *e UNUSED)
 {
-  CAST(io_backend, backend, a);
-  COMMAND_RETURN(c, make_connect_local(backend));
+  CAST(local_info, info, a);
+
+  io_connect_local(info,
+		   make_connect_continuation(NULL, c),
+		   e);
 }
 
 
