@@ -1,6 +1,6 @@
 /* io_commands.h
  *
- * $Id: io_commands.h,v 1.19 2001/07/06 20:10:09 nisse Exp $
+ * $Id: io_commands.h,v 1.23 2001/09/27 12:10:33 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -37,65 +37,40 @@
    (class
      (name io_write_file_info)
      (vars
-       (name . "const char *")
+       (name string)
        (flags . int)
        (mode . int)
        (block_size . UINT32)))
 */
 
 struct io_write_file_info *
-make_io_write_file_info(const char *name, int flags, int mode, UINT32 block_size);
+make_io_write_file_info(struct lsh_string *name, int flags, int mode, UINT32 block_size);
 
 extern struct command io_write_file_command;
 
 #define IO_WRITE_FILE (&io_write_file_command.super)
 
-/* Read a certain fd */
-
-/* GABA:
-   (class
-     (name io_read_fd)
-     (super command)
-     (vars
-       (fd . int)))
-*/
-
-void do_io_read_fd(struct command *s,
-		   struct lsh_object *a,
-		   struct command_continuation *c,
-		   struct exception_handler *e);
-
-#define STATIC_IO_READ_FD(fd) \
-{ STATIC_COMMAND(do_io_read_fd), fd }
-
-extern struct io_read_fd io_read_stdin;
-#define IO_READ_STDIN (&io_read_stdin.super.super)
-
 
 struct command *
-make_listen_with_callback(struct command *callback,
-			  struct io_backend *backend);
+make_listen_with_callback(struct command *callback);
 
-extern struct command_3 listen_with_callback;
+extern struct command_2 listen_with_callback;
 #define LISTEN_CALLBACK (&listen_with_callback.super.super)
 
 struct command *
-make_connect_port(struct io_backend *backend,
-		  struct address_info *target);
+make_connect_port(struct address_info *target);
+
+extern struct command_2 connect_connection_command;
+#define CONNECT_CONNECTION (&connect_connection_command.super.super)
+
+extern struct command connect_simple_command;
+#define CONNECT_SIMPLE (&connect_simple_command.super)
 
 struct command *
-make_connect_connection(struct io_backend *backend);
+make_listen_local(struct local_info *info);
 
 struct command *
-make_simple_connect(struct io_backend *backend,
-		    struct resource_list *resources);
-
-struct command *
-make_listen_local(struct io_backend *backend,
-		  struct local_info *info);
-
-struct command *
-make_connect_local(struct io_backend *backend);
+make_connect_local(void);
 
 extern struct command connect_local_command;
 #define CONNECT_LOCAL (&connect_local_command.super)
