@@ -2,7 +2,7 @@
  *
  * client main program
  *
- * $Id: lsh.c,v 1.146 2001/09/01 23:01:28 nisse Exp $ */
+ * $Id: lsh.c,v 1.155 2001/10/03 16:12:28 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -46,7 +46,7 @@
 #include "randomness.h"
 #include "rsa.h"
 #include "sexp.h"
-#include "spki_commands.h"
+#include "spki.h"
 #include "srp.h" 
 #include "ssh.h"
 #include "tcpforward_commands.h"
@@ -74,9 +74,6 @@ struct command_2 lsh_verifier_command;
 struct command_2 lsh_login_command;
 #define LSH_LOGIN (&lsh_login_command.super.super)
 
-static struct command options2known_hosts;
-#define OPTIONS2KNOWN_HOSTS (&options2known_hosts.super)
-
 struct command options2service;
 #define OPTIONS2SERVICE (&options2service.super)
 
@@ -131,12 +128,11 @@ STATIC_REQUEST_SERVICE(ATOM_SSH_USERAUTH);
 
 
 static struct lsh_options *
-make_options(struct io_backend *backend,
-	     struct exception_handler *handler,
+make_options(struct exception_handler *handler,
 	     int *exit_code)
 {
   NEW(lsh_options, self);
-  init_client_options(&self->super, backend,
+  init_client_options(&self->super, 
 		      make_default_random(NULL, handler),
 		      handler, exit_code);
 
@@ -182,91 +178,210 @@ DEFINE_COMMAND(options2service)
 
 /* Open hostkey database. By default, "~/.lsh/known_hosts". */
 
-static void
-do_options2known_hosts(struct command *ignored UNUSED,
-		       struct lsh_object *a,
-		       struct command_continuation *c,
-		       struct exception_handler *e)
+static struct spki_context *
+read_known_hosts(struct lsh_options *options)
 {
-  CAST(lsh_options, options, a);
-  
   struct lsh_string *tmp = NULL;
+  struct lsh_string *contents;
   const char *s = NULL;
-  struct lsh_fd *f;
+  int fd;
+  struct simple_buffer buffer;
+  struct spki_context *context;
+
+  context = make_spki_context(options->signature_algorithms);
   
   if (options->known_hosts)
     s = options->known_hosts;
-  else 
+  else
     {
       tmp = ssh_format("%lz/.lsh/known_hosts", options->home);
       s = lsh_get_cstring(tmp);
     }
-  
-  f = io_read_file(options->super.backend, s, e);
 
-  if (!f)
+  fd = open(s, O_RDONLY);
+  if (fd < 0)
     {
-      werror("Failed to open '%z' (errno = %i): %z.\n",
-	     s, errno, STRERROR(errno));
-      COMMAND_RETURN(c, make_spki_context(options->signature_algorithms));
-    }
-  else
-    {
-      CAST_SUBTYPE(command, read,
-		   make_spki_read_acls(options->signature_algorithms));
-      COMMAND_CALL(read, f, c, e);
+      lsh_string_free(tmp);
+      werror("Failed to open `%z' for reading: %z\n",
+             s, STRERROR(errno));
+      return context;
     }
+
   lsh_string_free(tmp);
+
+  contents = io_read_file_raw(fd, 5000);
+  
+  if (!contents)
+  {
+    werror("Failed to read known_hosts file: %z\n",
+           STRERROR(errno));
+    close(fd);
+    return context;
+  }
+
+  close(fd);
+
+  simple_buffer_init(&buffer, contents->length, contents->data);
+  while (!parse_eod(&buffer))
+    {
+      struct sexp *expr = sexp_parse(SEXP_TRANSPORT, &buffer);
+
+      if (!expr)
+        {
+          werror("read_known_hosts: S-expression syntax error.\n");
+          break;
+        }
+      if (!spki_add_acl(context, expr))
+        {
+          werror("read_known_hosts: Invalid ACL.\n");
+          break;
+        }
+    }
+  lsh_string_free(contents);
+  return context;
 }
 
-static struct command options2known_hosts =
-STATIC_COMMAND(do_options2known_hosts);
+/* For now, supports only a single key */
+static struct object_list *
+read_user_keys(struct lsh_options *options)
+{
+  struct lsh_string *tmp = NULL;
+  struct lsh_string *contents;
+  const char *name = NULL;
+  struct sexp *expr;
+  int fd;
+  int algorithm_name;
+  
+  struct signer *s;
+  struct verifier *v;
+  struct lsh_string *spki_public;
+
+  trace("read-user_keys\n");
+  
+  if (!options->with_publickey)
+    return make_object_list(0, -1);
+
+  if (options->identity)
+    name = options->identity;
+  else 
+    {
+      tmp = ssh_format("%lz/.lsh/identity", options->home);
+      name = lsh_get_cstring(tmp);
+    }
+
+  fd = open(name, O_RDONLY);
+  if (fd < 0)
+    {
+      werror("Failed to open `%z' for reading: %z\n",
+             name, STRERROR(errno));
+      lsh_string_free(tmp);
+
+      return make_object_list(0, -1);
+    }
+
+  lsh_string_free(tmp);
+
+  contents = io_read_file_raw(fd, 2000);
+
+  if (!contents)
+    {
+      werror("Failed to read private key file: %z\n",
+             STRERROR(errno));
+      close(fd);
+
+      return make_object_list(0, -1);
+    }
+
+  close(fd);
+
+  expr = string_to_sexp(SEXP_TRANSPORT, contents, 1);
+
+  if (!expr)
+    {
+      werror("S-expression syntax error in private key.\n");
+      return make_object_list(0, -1);
+    }
+
+  if (options->super.tty)
+    {
+      struct alist *mac = make_alist(0, -1);
+      struct alist *crypto = make_alist(0, -1);
+
+      alist_select_l(mac, options->algorithms->algorithms,
+		     2, ATOM_HMAC_SHA1, ATOM_HMAC_MD5, -1);
+      alist_select_l(crypto, options->algorithms->algorithms,
+		     3, ATOM_3DES_CBC, ATOM_BLOWFISH_CBC,
+		     ATOM_RIJNDAEL_CBC_LOCAL, -1);
+      
+      expr = spki_pkcs5_decrypt(mac, crypto,
+                                options->super.tty,
+                                expr);
+      if (!expr)
+        {
+          werror("Decrypting private key failed.\n");
+          return make_object_list(0, -1);
+        }
+    }
+  
+  s = spki_sexp_to_signer(options->signature_algorithms, expr,
+                          &algorithm_name);
+  if (!s)
+    {
+      werror("Invalid private key.\n");
+      return make_object_list(0, -1);
+    }
+  
+  v = SIGNER_GET_VERIFIER(s);
+  assert(v);
+  
+  spki_public = sexp_format(spki_make_public_key(v),
+			    SEXP_CANONICAL, 0);
+
+  /* Test key here? */  
+  switch (algorithm_name)
+    {	  
+    case ATOM_DSA:
+      return make_object_list(2,
+                              make_keypair(ATOM_SSH_DSS,
+                                           PUBLIC_KEY(v),
+                                           s),
+                              make_keypair(ATOM_SPKI_SIGN_DSS,
+                                           spki_public, s),
+                              -1);
+      break;
+
+    case ATOM_RSA_PKCS1_SHA1:
+      return make_object_list(2, 
+                              make_keypair(ATOM_SSH_RSA,
+                                           PUBLIC_KEY(v),
+                                           s),
+                              make_keypair(ATOM_SPKI_SIGN_RSA,
+                                           spki_public, s),
+                              -1);
+
+    case ATOM_RSA_PKCS1_MD5:
+      return make_object_list(1,
+                              make_keypair(ATOM_SPKI_SIGN_RSA,
+                                           spki_public, s),
+                              -1);
+
+    default:
+      fatal("Internal error!\n");
+    }
+}
+
+/* FIXME: Call read_user_key directly from main instead. */
 
 /* Read user's private keys. By default, "~/.lsh/identity". */
 static void
 do_options2identities(struct command *ignored UNUSED,
 		      struct lsh_object *a,
 		      struct command_continuation *c,
-		      struct exception_handler *e)
+		      struct exception_handler *e UNUSED)
 {
   CAST(lsh_options, options, a);
-  
-  struct lsh_string *tmp = NULL;
-  const char *s = NULL;
-  struct lsh_fd *f = NULL;
 
-  trace("do_options2identities\n");
-  
-  if (!options->with_publickey)
-    {
-      COMMAND_RETURN(c, make_object_list(0, -1));
-      return;
-    }
-  if (options->identity)
-    s = options->identity;
-  else 
-    {
-      tmp = ssh_format("%lz/.lsh/identity", options->home);
-      s = lsh_get_cstring(tmp);
-    }
-  
-  f = io_read_file(options->super.backend, s, e);
-  
-  if (!f)
-    {
-      werror("Failed to open '%z' (errno = %i): %z.\n",
-	     s, errno, STRERROR(errno));
-      COMMAND_RETURN(c, make_object_list(0, -1));
-    }
-  else
-    {
-      struct command *command
-	= make_spki_read_userkeys(options->algorithms->algorithms,
-				  options->signature_algorithms,
-				  options->super.tty);
-      COMMAND_CALL(command, f, c, e);
-    }
-  lsh_string_free(tmp);
+  COMMAND_RETURN(c, read_user_keys(options));
 }
 
 static struct command options2identities =
@@ -543,9 +658,9 @@ DEFINE_COMMAND2(lsh_login_command)
    (expr
      (name make_lsh_connect)
      (params
-       (connect object command)
        (handshake object handshake_info)
-       (init object make_kexinit))
+       (init object make_kexinit)
+       (db object lookup_verifier))
      (expr (lambda (options)
                ; What to do with the service
 	       ((progn (options2actions options))
@@ -557,10 +672,9 @@ DEFINE_COMMAND2(lsh_login_command)
 		     ; Start the ssh transport protocol
 	             (connection_handshake
 		       handshake init
-		       (options2verifier options
-				         (options2known_hosts options))
+		       db
  		       ; Connect using tcp
-		       (connect (options2remote options)))))))))
+		       (connect_simple (options2remote options)))))))))
 */
 
 
@@ -616,19 +730,16 @@ main_options[] =
     "Enable DH support (default, unless SRP is being used).", 0 },
 
   { "no-dh-keyexchange", OPT_DH | ARG_NOT, NULL, 0, "Disable DH support.", 0 },
-
-#if 0
-  { "userauth", OPT_USERAUTH, NULL, 0,
-    "Request the ssh-userauth service (default, unless SRP is being used).", 0 },
-  { "no-userauth", OPT_USERAUTH | ARG_NOT, NULL, 0,
-    "Request the ssh-userauth service (default if SRP is used).", 0 },
-#endif
   
   /* Actions */
   { "forward-remote-port", 'R', "remote-port:target-host:target-port",
     0, "", CLIENT_ARGP_ACTION_GROUP },
   { "gateway", 'G', NULL, 0, "Setup a local gateway", 0 },
 
+  /* FIXME: Perhaps this should be moved from lsh.c to client.c? It
+   * doesn't work with lshg. Or perhaps that can be fixed?
+   * About the same problem applies to -R. */
+  
   { "x11-forward", 'x', NULL, 0, "Enable X11 forwarding.", CLIENT_ARGP_MODIFIER_GROUP },
   { "no-x11-forward", 'x' | ARG_NOT, NULL, 0,
     "Disable X11 forwarding (default).", 0 },
@@ -748,7 +859,7 @@ main_argp_parser(int key, char *arg, struct argp_state *state)
 	if (s)
 	  {
 	    struct lsh_fd *f
-	      = io_write_file(self->super.backend, s,
+	      = io_write_file(s,
 			      O_CREAT | O_APPEND | O_WRONLY,
 			      0600, 500, NULL,
 			      make_report_exception_handler
@@ -792,7 +903,7 @@ main_argp_parser(int key, char *arg, struct argp_state *state)
 	      
 	  client_add_action(&self->super,
 			    make_gateway_setup
-			    (make_listen_local(self->super.backend, gateway)));
+			    (make_listen_local(gateway)));
 	}
 
       if (object_queue_is_empty(&self->super.actions))
@@ -840,8 +951,7 @@ main_argp_parser(int key, char *arg, struct argp_state *state)
 	  argp_error(state, "Invalid forward specification '%s'.", arg);
 
 	client_add_action(&self->super, make_forward_remote_port
-			  (self->super.backend,
-			   make_address_info((self->super.with_remote_peers
+			  (make_address_info((self->super.with_remote_peers
 					      ? ssh_format("%lz", "0.0.0.0")
 					      : ssh_format("%lz", "127.0.0.1")),
 					     listen_port),
@@ -931,7 +1041,8 @@ make_lsh_default_handler(int *status, struct exception_handler *parent,
 int main(int argc, char **argv)
 {
   struct lsh_options *options;
-
+  struct spki_context *spki;
+  
   /* Default exit code if something goes wrong. */
   int lsh_exit_code = 17;
 
@@ -939,7 +1050,7 @@ int main(int argc, char **argv)
     = make_lsh_default_handler(&lsh_exit_code, &default_exception_handler,
 			       HANDLER_CONTEXT);
 
-  struct io_backend *backend = make_io_backend();
+  io_init();
   
   /* For filtering messages. Could perhaps also be used when converting
    * strings to and from UTF8. */
@@ -948,14 +1059,15 @@ int main(int argc, char **argv)
   /* FIXME: Choose character set depending on the locale */
   set_local_charset(CHARSET_LATIN1);
 
-  options = make_options(backend, handler, &lsh_exit_code);
+  options = make_options(handler, &lsh_exit_code);
 
   argp_parse(&main_argp, argc, argv, ARGP_IN_ORDER, NULL, options);
+
+  spki = read_known_hosts(options);
   
   {
     struct lsh_object *o =
       make_lsh_connect(
-	make_simple_connect(backend, NULL),
 	make_handshake_info(CONNECTION_CLIENT,
 			    "lsh - a free ssh", NULL,
 			    SSH_MAX_PACKET,
@@ -968,7 +1080,12 @@ int main(int argc, char **argv)
 			    options->algorithms->crypto_algorithms,
 			    options->algorithms->mac_algorithms,
 			    options->algorithms->compression_algorithms,
-			    make_int_list(0, -1)));
+			    make_int_list(0, -1)),
+        make_lsh_host_db(spki,
+                         options->super.tty,
+                         options->super.remote,
+                         options->sloppy,
+                         options->capture_file));
     
     CAST_SUBTYPE(command, lsh_connect, o);
 
@@ -986,13 +1103,11 @@ int main(int argc, char **argv)
     werror("Strange: Final redirect of stdout to /dev/null failed.\n");
 #endif
   
-  io_run(backend);
+  io_run();
 
   /* Close all files and other resources associated with the backend. */
-  io_final(backend);
+  io_final();
   
-  gc_final();
-
   /* FIXME: Perhaps we have to reset the stdio file descriptors to
    * blocking mode? */
   return lsh_exit_code;
