@@ -1,6 +1,6 @@
 /* lsh.h
  *
- * $Id: lsh.h,v 1.21 2001/09/01 10:38:54 nisse Exp $ */
+ * $Id: lsh.h,v 1.23 2001/10/02 22:41:06 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -104,7 +104,8 @@ struct lsh_string
 
 /* Forward declarations of various structures */
 
-/* abstract_crypto.h */
+/* abstract_io.h */
+struct abstract_write;
 
 /* client_x11.c */
 struct client_x11_display;
@@ -117,7 +118,6 @@ struct interact;
 
 /* io.h */
 struct address_info;
-struct io_backend;
 struct listen_value;
 struct lsh_fd;
 
@@ -132,6 +132,9 @@ struct randomness;
 struct exit_callback;
 struct reap;
 
+/* resource.h */
+struct resource;
+
 /* sexp.h */
 struct sexp;
 struct sexp_iterator;
