@@ -2,7 +2,7 @@
  *
  * main proxy program.
  *
- * $Id: lsh_proxy.c,v 1.23 2001/07/06 20:19:36 nisse Exp $ */
+ * $Id: lsh_proxy.c,v 1.28 2001/10/03 16:12:38 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -55,9 +55,7 @@
 #include "server_session.h"
 #include "server_userauth.h"
 #include "sexp.h"
-#include "sexp_commands.h"
 #include "spki.h"
-#include "spki_commands.h"
 #include "ssh.h"
 #include "version.h"
 #include "werror.h"
@@ -69,8 +67,8 @@
 struct command options2local;
 #define OPTIONS2LOCAL (&options2local.super)
 
-struct command options2keyfile;
-#define OPTIONS2KEYFILE (&options2keyfile.super)
+struct command options2keys;
+#define OPTIONS2KEYS (&options2keys.super)
 
 struct command options2signature_algorithms;
 #define OPTIONS2SIGNATURE_ALGORITHMS \
@@ -123,7 +121,6 @@ const char *argp_program_bug_address = BUG_ADDRESS;
      (name lsh_proxy_options)
      (super algorithms_options)
      (vars
-       (backend object io_backend)
        (random object randomness_with_poll)
        (signature_algorithms object alist)
        (style . sexp_argp_state)
@@ -143,8 +140,7 @@ const char *argp_program_bug_address = BUG_ADDRESS;
 */
 
 static struct lsh_proxy_options *
-make_lsh_proxy_options(struct io_backend *backend, 
-		       struct randomness_with_poll *random, 
+make_lsh_proxy_options(struct randomness_with_poll *random, 
 		       struct alist *algorithms)
 {
   NEW(lsh_proxy_options, self);
@@ -154,7 +150,6 @@ make_lsh_proxy_options(struct io_backend *backend,
     = make_alist(1,
 		 ATOM_DSA, make_dsa_algorithm(&random->super), -1);
 
-  self->backend = backend;
   self->random = random;
   self->style = SEXP_TRANSPORT;
   self->interface = NULL;
@@ -205,28 +200,22 @@ DEFINE_COMMAND(options2signature_algorithms)
 }
 
 /* Read server's private key */
-DEFINE_COMMAND(options2keyfile)
+/* FIXME: Call read_host_key directly from main instead. */
+DEFINE_COMMAND(options2keys)
      (struct command *ignored UNUSED,
       struct lsh_object *a,
       struct command_continuation *c,
-      struct exception_handler *e)
+      struct exception_handler *e UNUSED)
 {
   CAST(lsh_proxy_options, options, a);
-  
-  struct lsh_fd *f;
 
-  f = io_read_file(options->backend, options->hostkey, e);
+  struct alist *keys = make_alist(0, -1);
+  read_host_key(options->hostkey, options->signature_algorithms, keys);
 
-  if (f)
-    COMMAND_RETURN(c, f);
-  else
-    {
-      werror("Failed to open '%z' (errno = %i): %z.\n",
-	     options->hostkey, errno, STRERROR(errno));
-      EXCEPTION_RAISE(e, make_io_exception(EXC_IO_OPEN_READ, NULL, errno, NULL));
-    }
+  COMMAND_RETURN(c, keys);
 }
 
+
 static const struct argp_option
 main_options[] =
 {
@@ -477,7 +466,6 @@ DEFINE_COMMAND2(proxy_destination)
    (expr
      (name lsh_proxy_listen)
      (params
-       (backend object io_backend)
        (services object command)
        (handshake_server object command)
        (handshake_client object command))
@@ -493,7 +481,6 @@ DEFINE_COMMAND2(proxy_destination)
 	         (handshake_server options)   ; callback to perform server side handshake
 		 (handshake_client options)   ; callback to perform client side handshake
 		 client_addr)))
-	   backend 
 	   (options2local options)))) )
 
 	     
@@ -509,8 +496,7 @@ DEFINE_COMMAND2(proxy_destination)
        (lambda (options client_addr)
          (connection_handshake    ; handshake on the client side
             handshake init
-	    (spki_read_hostkeys (options2signature_algorithms options)
-	                        (options2keyfile options)) 
+            (options2keys options)
 	    (log_peer client_addr)))))
 
 */
@@ -519,7 +505,6 @@ DEFINE_COMMAND2(proxy_destination)
    (expr
      (name lsh_proxy_handshake_server)
      (params
-       (connect object command)
        (verifier object lookup_verifier)
        (handshake object handshake_info)
        (init object make_kexinit))
@@ -529,7 +514,7 @@ DEFINE_COMMAND2(proxy_destination)
 	   (connection_handshake 
              handshake init
              verifier 
-             (connect (proxy_destination options client_addr)))))))
+             (connect_simple (proxy_destination options client_addr)))))))
 */
 
 /* Invoked when the client requests the userauth service. */
@@ -577,16 +562,13 @@ int main(int argc, char **argv)
   struct make_kexinit *make_kexinit;
   struct exception_handler *handler;
 
-  struct io_backend *backend;
-
-  backend = make_io_backend();
-
+  io_init();
+  
 #if WITH_ALF
   proxy_alf_init();
   proxy_alf_end();
 #endif
 
-
   /* For filtering messages. Could perhaps also be used when converting
    * strings to and from UTF8. */
   setlocale(LC_CTYPE, "");
@@ -597,7 +579,7 @@ int main(int argc, char **argv)
     (make_report_exception_info(EXC_IO, EXC_IO, "lsh_proxy: "),
      &default_exception_handler,
      HANDLER_CONTEXT);
-  reaper = make_reaper(backend); 
+  reaper = make_reaper(); 
   r = make_default_random(reaper, handler);
 
   algorithms_server = all_symmetric_algorithms();
@@ -606,7 +588,7 @@ int main(int argc, char **argv)
   
   signature_algorithms = all_signature_algorithms(&r->super);
   
-  options = make_lsh_proxy_options(backend, r, algorithms_server);
+  options = make_lsh_proxy_options(r, algorithms_server);
   
   argp_parse(&main_argp, argc, argv, 0, NULL, options);
 
@@ -766,8 +748,7 @@ int main(int argc, char **argv)
       
     {
       struct lsh_object *o = lsh_proxy_listen
-	(backend,
-	 make_proxy_offer_service
+	(make_proxy_offer_service
 	 (make_alist(1, 
 		     ATOM_SSH_USERAUTH, 
 		     lsh_proxy_services
@@ -785,8 +766,7 @@ int main(int argc, char **argv)
 		      -1)),
 
 	 /* callback to call when client<->proxy handshake finished */
-	 (struct command *)lsh_proxy_handshake_server(make_simple_connect(backend, NULL),
-						      make_fake_host_db(),
+	 (struct command *)lsh_proxy_handshake_server(make_fake_host_db(),
 						      make_handshake_info
 						      (CONNECTION_CLIENT,
 						       "lsh_proxy_client - a free ssh",
@@ -816,7 +796,9 @@ int main(int argc, char **argv)
     }
   }
   
-  io_run(backend);
+  io_run();
 
+  io_final();
+  
   return 0;
 }
