@@ -2,7 +2,7 @@
  *
  * main server program.
  *
- * $Id: lshd.c,v 1.121 2001/08/26 14:48:18 nisse Exp $ */
+ * $Id: lshd.c,v 1.131 2001/10/03 16:12:49 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -47,8 +47,7 @@
 #include "server_pty.h"
 #include "server_session.h"
 #include "sexp.h"
-#include "sexp_commands.h"
-#include "spki_commands.h"
+#include "spki.h"
 #include "srp.h"
 #include "ssh.h"
 #include "tcpforward.h"
@@ -65,12 +64,14 @@
 struct command options2local;
 #define OPTIONS2LOCAL (&options2local.super)
 
-struct command options2keyfile;
-#define OPTIONS2KEYFILE (&options2keyfile.super)
+struct command options2keys;
+#define OPTIONS2KEYS (&options2keys.super)
 
+#if 0
 struct command options2signature_algorithms;
 #define OPTIONS2SIGNATURE_ALGORITHMS \
   (&options2signature_algorithms.super)
+#endif
 
 #include "lshd.c.x"
 
@@ -146,7 +147,6 @@ const char *argp_program_bug_address = BUG_ADDRESS;
      (name lshd_options)
      (super algorithms_options)
      (vars
-       (backend object io_backend)
        (e object exception_handler)
        
        (reaper object reap)
@@ -212,16 +212,15 @@ make_lshd_exception_handler(struct exception_handler *parent,
 }
 
 static struct lshd_options *
-make_lshd_options(struct io_backend *backend)
+make_lshd_options(void)
 {
   NEW(lshd_options, self);
 
   init_algorithms_options(&self->super, all_symmetric_algorithms());
 
-  self->backend = backend;
   self->e = make_lshd_exception_handler(&default_exception_handler,
 					HANDLER_CONTEXT);
-  self->reaper = make_reaper(backend);
+  self->reaper = make_reaper();
   self->random = make_default_random(self->reaper, self->e);
 
   self->signature_algorithms = all_signature_algorithms(&self->random->super);
@@ -288,28 +287,20 @@ DEFINE_COMMAND(options2signature_algorithms)
   COMMAND_RETURN(c, options->signature_algorithms);
 }
 
-/* Read server's private key */
 
-DEFINE_COMMAND(options2keyfile)
+/* FIXME: Call read_host_key directly from main instead. */
+DEFINE_COMMAND(options2keys)
      (struct command *ignored UNUSED,
       struct lsh_object *a,
       struct command_continuation *c,
-      struct exception_handler *e)
+      struct exception_handler *e UNUSED)
 {
   CAST(lshd_options, options, a);
-  
-  struct lsh_fd *f;
 
-  f = io_read_file(options->backend, options->hostkey, e);
+  struct alist *keys = make_alist(0, -1);
+  read_host_key(options->hostkey, options->signature_algorithms, keys);
 
-  if (f)
-    COMMAND_RETURN(c, f);
-  else
-    {
-      werror("Failed to open '%z' (errno = %i): %z.\n",
-	     options->hostkey, errno, STRERROR(errno));
-      EXCEPTION_RAISE(e, make_io_exception(EXC_IO_OPEN_READ, NULL, errno, NULL));
-    }
+  COMMAND_RETURN(c, keys);
 }
 
 
@@ -466,7 +457,7 @@ main_argp_parser(int key, char *arg, struct argp_state *state)
 	struct user_db *user_db = NULL;
 	
 	if (self->with_password || self->with_publickey || self->with_srp_keyexchange)
-	  user_db = make_unix_user_db(self->backend, self->reaper,
+	  user_db = make_unix_user_db(self->reaper,
 				      self->pw_helper, self->login_shell,
 				      self->allow_root);
 	  
@@ -698,14 +689,11 @@ main_argp =
    (expr
      (name make_lshd_listen)
      (params
-       (backend object io_backend)
        (handshake object handshake_info)
        (init object make_kexinit)
        (services object command) )
      (expr (lambda (options)
-             (let ((keys 
-		    (spki_read_hostkeys (options2signature_algorithms options)
-			                (options2keyfile options))))
+             (let ((keys (options2keys options)))
 	       (listen_callback
 	         (lambda (lv)
     		   (services (connection_handshake
@@ -713,7 +701,6 @@ main_argp =
     				  (kexinit_filter init keys)
     				  keys 
     				  (log_peer lv))))
-		 backend
 		 (options2local options))))))
 */
 
@@ -735,23 +722,13 @@ main_argp =
 */
 
 #if WITH_GCOV
-/* FIXME: Perhaps move to daemon.c? */
 /* Catch SIGTERM and call exit(). That way, profiling info is written
  * properly when the process is terminated. */
 
-static volatile sig_atomic_t terminate;
-
-static void terminate_handler(int signum)
-{
-  assert(signum == SIGTERM);
-
-  terminate = 1;
-}
-
 static void
 do_terminate_callback(struct lsh_callback *s UNUSED)
 {
-  gc_final();
+  io_final();
   exit(0);
 }
 
@@ -760,33 +737,21 @@ terminate_callback =
 { STATIC_HEADER, do_terminate_callback };
 
 static void
-install_terminate_handler(struct io_backend *backend)
+install_terminate_handler(void)
 {
-  struct sigaction term;
-  memset(&term, 0, sizeof(term));
-
-  term.sa_handler = terminate_handler;
-  sigemptyset(&term.sa_mask);
-  term.sa_flags = 0;
-
-  if (sigaction(SIGTERM, &term, NULL) < 0)
-    {
-      werror ("Failed to install SIGTERM handler (errno = %i): %z\n",
-	      errno, STRERROR(errno));
-      exit(EXIT_FAILURE);
-    }
-  io_signal_handler(backend, &terminate, &terminate_callback);
+  io_signal_handler(SIGTERM, &terminate_callback);
 }
+
 #endif /* WITH_GCOV */
 
 int main(int argc, char **argv)
 {
   struct lshd_options *options;
 
-  struct io_backend *backend = make_io_backend();
+  io_init();
 
 #if WITH_GCOV
-  install_terminate_handler(backend);
+  install_terminate_handler();
 #endif
   
   /* For filtering messages. Could perhaps also be used when converting
@@ -796,11 +761,11 @@ int main(int argc, char **argv)
   /* FIXME: Choose character set depending on the locale */
   set_local_charset(CHARSET_LATIN1);
 
-  options = make_lshd_options(backend);
+  options = make_lshd_options();
   
   trace("Parsing options...\n");
   argp_parse(&main_argp, argc, argv, 0, NULL, options);
-  trace("Parsing options...\n");  
+  trace("Parsing options... done\n");  
 
   if (!options->corefile && !daemon_disable_core())
     {
@@ -808,30 +773,56 @@ int main(int argc, char **argv)
       return EXIT_FAILURE;
     }
 
-  if (options->daemonic && !options->no_syslog)
-    {
-#if HAVE_SYSLOG
-      set_error_syslog("lshd");
-#else /* !HAVE_SYSLOG */
-      werror("lshd: No syslog. Further messages will be directed to /dev/null.\n");
-#endif /* !HAVE_SYSLOG */
-    }
-
   if (options->daemonic)
-    switch (daemon_init())
-      {
-      case 0:
-	werror("lshd: Spawning into background failed.\n");
-	return EXIT_FAILURE;
-      case DAEMON_INETD:
-	werror("lshd: spawning from inetd not yet supported.\n");
-	return EXIT_FAILURE;
-      case DAEMON_INIT:
-      case DAEMON_NORMAL:
-	break;
-      default:
-	fatal("Internal error\n");
-      }
+    {
+      if (options->no_syslog)
+        {
+          /* Just put process into the background. --no-syslog is an
+           * inappropriate name */
+          switch (fork())
+            {
+            case 0:
+              /* Child */
+              /* FIXME: Should we create a new process group, close our tty
+               * and stdio, etc? */
+              trace("forked into background. New pid: %i.\n", getpid());
+              break;
+              
+            case -1:
+              /* Error */
+              werror("background_process: fork failed (errno = %i): %z\n",
+                     errno, STRERROR(errno));
+              break;
+              
+            default:
+              /* Parent */
+              _exit(EXIT_SUCCESS);
+            }
+        }
+      else
+        {
+#if HAVE_SYSLOG
+          set_error_syslog("lshd");
+#else /* !HAVE_SYSLOG */
+          werror("lshd: No syslog. Further messages will be directed to /dev/null.\n");
+#endif /* !HAVE_SYSLOG */
+
+          switch (daemon_init())
+            {
+            case 0:
+              werror("lshd: Spawning into background failed.\n");
+              return EXIT_FAILURE;
+            case DAEMON_INETD:
+              werror("lshd: spawning from inetd not yet supported.\n");
+              return EXIT_FAILURE;
+            case DAEMON_INIT:
+            case DAEMON_NORMAL:
+              break;
+            default:
+              fatal("Internal error\n");
+            }
+        }
+    }
   
   if (options->use_pid_file && !daemon_pidfile(options->pid_file))
     {
@@ -854,21 +845,24 @@ int main(int argc, char **argv)
     /* Supported channel requests */
     struct alist *supported_channel_requests
       = make_alist(2,
-		   ATOM_SHELL, make_shell_handler(backend),
-		   ATOM_EXEC, make_exec_handler(backend),
+		   ATOM_SHELL, &shell_request_handler,
+		   ATOM_EXEC, &exec_request_handler,
 		   -1);
     
 #if WITH_PTY_SUPPORT
     if (options->with_pty)
-      ALIST_SET(supported_channel_requests,
-		ATOM_PTY_REQ, &pty_request_handler.super);
+      {
+        ALIST_SET(supported_channel_requests,
+                  ATOM_PTY_REQ, &pty_request_handler.super);
+        ALIST_SET(supported_channel_requests,
+                  ATOM_WINDOW_CHANGE, &window_change_request_handler.super);
+      }
 #endif /* WITH_PTY_SUPPORT */
 
     if (options->subsystems)
       ALIST_SET(supported_channel_requests,
 		ATOM_SUBSYSTEM,
-		&make_subsystem_handler(backend,
-					options->subsystems)->super);
+		&make_subsystem_handler(options->subsystems)->super);
 		
     session_setup = make_install_fix_channel_open_handler
       (ATOM_SESSION, make_open_session(supported_channel_requests));
@@ -878,26 +872,21 @@ int main(int argc, char **argv)
       connection_hooks = make_object_list
 	(4,
 	 session_setup,
-	 make_tcpip_forward_hook(backend),
+	 make_tcpip_forward_hook(),
 	 make_install_fix_global_request_handler
 	 (ATOM_CANCEL_TCPIP_FORWARD, &tcpip_cancel_forward),
-	 make_direct_tcpip_hook(backend),
+	 make_direct_tcpip_hook(),
 	 -1);
     else
 #endif
       connection_hooks
 	= make_object_list (1, session_setup, -1);
     {
-      /* FIXME: We should check that we have at least one host key. We
-       * should also extract the host-key algorithms for which we have
-       * keys, instead of hardcoding ssh-dss below. */
-
       CAST_SUBTYPE(command, connection_service,
 		   make_lshd_connection_service(connection_hooks));
       CAST_SUBTYPE(command, server_listen, 		   
 		   make_lshd_listen
-		   (backend,
-		    make_handshake_info(CONNECTION_SERVER,
+		   (make_handshake_info(CONNECTION_SERVER,
 					"lsh - a free ssh",
 					NULL,
 					SSH_MAX_PACKET,
@@ -931,9 +920,9 @@ int main(int argc, char **argv)
     }
   }
   
-  io_run(backend);
+  io_run();
 
-  gc_final();
+  io_final();
   
   return 0;
 }
