@@ -1,6 +1,6 @@
 /* proxy_x11forward.c
  *
- * $Id: proxy_x11forward.c,v 1.2 2000/03/26 20:38:34 nisse Exp $ */
+ * $Id: proxy_x11forward.c,v 1.5 2001/04/04 21:50:48 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -35,9 +35,7 @@
 static void
 do_proxy_open_x11(struct channel_open *s UNUSED,
 		  struct ssh_connection *connection,
-		  UINT32 type,
-		  UINT32 send_window_size,
-		  UINT32 send_max_packet,
+		  struct channel_open_info *info,
 		  struct simple_buffer *args,
 		  struct command_continuation *c,
 		  struct exception_handler *e)
@@ -62,15 +60,13 @@ do_proxy_open_x11(struct channel_open *s UNUSED,
       /* NOTE: The origin's rec_window_size and rec_max_packet becomes the target's
        * send_window_size and send_max_packet. */
       if (connection->chain->peer_flags & PEER_X11_OPEN_KLUDGE)
-	o = make_gateway_channel_open_command(type, 
-					      send_window_size, send_max_packet,
+	o = make_gateway_channel_open_command(info,
 					      ssh_format("%S",
 							 host), 
 					      NULL);
       else
 	/* FIXME: maybe parse the sent string to get the port value */
-	o = make_gateway_channel_open_command(type, 
-					      send_window_size, send_max_packet,
+	o = make_gateway_channel_open_command(info,
 					      ssh_format("%S%i",
 							 host, port), 
 					      NULL);
@@ -80,15 +76,12 @@ do_proxy_open_x11(struct channel_open *s UNUSED,
 	werror("datafellows compatible x11 open request: %S\n", host);
       COMMAND_CALL(o,
 		   connection->chain,
-		   make_gateway_channel_open_continuation(c, server),
+		   make_gateway_channel_open_continuation(c, NULL, server),
 		   e);
 
     }
   else
-    {
-      PROTOCOL_ERROR(e, "Trailing garbage in open message");
-    }
-  lsh_string_free(host);
+    PROTOCOL_ERROR(e, "Trailing garbage in open message");
 }
 
 struct channel_open *
