@@ -2,7 +2,7 @@
  *
  *
  *
- * $Id: publickey_crypto.h,v 1.42 2000/09/19 18:28:07 nisse Exp $ */
+ * $Id: publickey_crypto.h,v 1.43 2000/11/26 23:25:57 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -59,10 +59,10 @@ make_keypair(UINT32 type,
        (order bignum)
        (generator bignum)
        
-       ;; Checks if a bignum is in the correct range for being a group element. 
+       ; Checks if a bignum is in the correct range for being a group element. 
        (range method int "mpz_t x")
 
-       ; (member method int "mpz_t x")
+       ;; (member method int "mpz_t x")
        
        (invert method void "mpz_t res" "mpz_t x")
        (combine method void "mpz_t res" "mpz_t a" "mpz_t b")
