@@ -2,7 +2,7 @@
  *
  * A read handler for application data.
  *
- * $Id: read_data.h,v 1.13 2000/02/15 18:30:08 nisse Exp $ */
+ * $Id: read_data.h,v 1.15 2000/04/21 22:29:41 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -30,14 +30,6 @@
 
 struct io_callback *
 make_read_data(struct ssh_channel *channel,
-	       UINT32 overhead,
 	       struct abstract_write *write);
 
-#if 0
-struct exception_handler *
-make_exc_read_eof_channel_handler(struct ssh_channel *channel,
-				  struct exception_handler *e,
-				  const char *context);
-#endif
-
 #endif /* LSH_READ_DATA_H_INCLUDED */
