@@ -2,7 +2,7 @@
  *
  * Handle child processes.
  *
- * $Id: reaper.c,v 1.18 2001/02/20 17:49:20 nisse Exp $
+ * $Id: reaper.c,v 1.22 2001/09/27 12:12:01 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -44,15 +44,6 @@
 
 #include "reaper.c.x"
 
-static volatile sig_atomic_t halloween;
-
-static void child_handler(int signum)
-{
-  assert(signum == SIGCHLD);
-
-  halloween = 1;
-}
-
 /* GABA:
    (class
      (name reaper)
@@ -140,7 +131,7 @@ do_reaper_callback(struct lsh_callback *s)
       else switch(errno)
 	{
 	case EINTR:
-	  werror("reaper.c: waitpid() returned EINTR.\n");
+	  werror("reaper.c: waitpid returned EINTR.\n");
 	  break;
 	case ECHILD:
 	  /* No more child processes */
@@ -162,34 +153,15 @@ make_reaper_callback(struct reaper *reaper)
   return &self->super;
 }
 
-static void
-reaper_install_handler(struct reaper *reaper,
-		       struct io_backend *b)
-{
-  struct sigaction chld;
-  memset(&chld, 0, sizeof(chld));
-
-  chld.sa_handler = child_handler;
-  sigemptyset(&chld.sa_mask);
-  chld.sa_flags = SA_NOCLDSTOP;
-  
-  halloween = 0;
-
-  if (sigaction(SIGCHLD, &chld, NULL) < 0)
-    fatal("Failed to install handler for SIGCHLD.\n");
-
-  io_signal_handler(b, &halloween, make_reaper_callback(reaper));
-}
-
 struct reap *
-make_reaper(struct io_backend *b)
+make_reaper(void)
 {
   NEW(reaper, self);
 
   self->super.reap = do_reap;
   self->children = make_linked_alist(0, -1);
 
-  reaper_install_handler(self, b);
+  io_signal_handler(SIGCHLD, make_reaper_callback(self));
 
   return &self->super;
 }
