@@ -2,7 +2,7 @@
  *
  * Handle child processes.
  *
- * $Id: reaper.h,v 1.8 2000/11/15 17:21:36 nisse Exp $
+ * $Id: reaper.h,v 1.10 2001/09/24 16:51:13 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -52,10 +52,6 @@
 
 #define REAP(r, p, c) ((r)->reap((r), (p), (c)))
 
-struct reap *make_reaper(struct io_backend *b);
-
-#if 0
-void reaper_run(struct reap *r, struct io_backend *b);
-#endif
+struct reap *make_reaper(void);
 
 #endif /* LSH_REAPER_H_INCLUDED */
