@@ -2,60 +2,49 @@
  *
  * The receive end of the rsync algorithm. 
  *
- * $Id: receive.c,v 1.2 2000/06/26 20:38:03 nisse Exp $ */
+ * $Id: receive.c,v 1.5 2001/06/13 16:50:22 nisse Exp $ */
 
 #include "rsync.h"
 
 #include <assert.h>
 #include <string.h>
 
-/* Reading a partial token */
-#define STATE_TOKEN 0
-
-/* Reading a literal */
-#define STATE_LITERAL 1
-
-/* Copying a local block */
-#define STATE_LOOKUP 2
-
-/* Reading final md5 sum */
-#define STATE_CHECKSUM 3
-
-/* Results in error */
-#define STATE_INVALID 4
-
+/* Hash the data after it is copied to the output buffer. */
 static void
 rsync_update(struct rsync_receive_state *s,
 	     UINT32 length)
 {
-  md5_update(&s->full_sum, s->next_in, length);
-  s->next_in += length;
-  s->avail_in -= length;
+  md5_update(&s->sum_md5, length, s->next_out);
+  s->next_out += length;
+  s->avail_out -= length;
 }
 
 #define GET() (assert(s->avail_in), s->avail_in--, *s->next_in++)
 
-int
+enum rsync_result_t
 rsync_receive(struct rsync_receive_state *s)
 {
   int progress = 0;
   
   for (;;)
+    /* FIXME: Cast to enum rsync_receive_mode, for better type checking? */
     switch (s->state)
       {
       do_token:
 	/* Here, i is octets read */
 	s->token = 0;
 	s->i = 0;
-	s->state = STATE_TOKEN;
-      case STATE_TOKEN:
+	s->state = RSYNC_READ_TOKEN;
+      case RSYNC_READ_TOKEN:
 	if (!s->avail_in)
 	  return progress ? RSYNC_PROGRESS : RSYNC_BUF_ERROR;
 	
 	s->token = (s->token << 8) | GET();
+	s->i++;
+
 	progress = 1;
 	
-	if (++s->i == 4)
+	if (s->i == 4)
 	  {
 	    if (!s->token)
 	      goto do_checksum;
@@ -68,7 +57,7 @@ rsync_receive(struct rsync_receive_state *s)
 	    else
 	      {
 		/* Index is one's complement */ 
-		s->token = -(s->token + 1);
+		s->token = ~s->token;
 		goto do_lookup;
 	      }
 	  }
@@ -76,8 +65,8 @@ rsync_receive(struct rsync_receive_state *s)
 
       do_literal:
 	/* Here, i is the number of octets to read. */
-	s->state = STATE_LITERAL;
-      case STATE_LITERAL:
+	s->state = RSYNC_READ_LITERAL;
+      case RSYNC_READ_LITERAL:
 	{
 	  UINT32 avail = MIN(s->avail_in, s->avail_out);
 	  if (!avail)
@@ -87,21 +76,25 @@ rsync_receive(struct rsync_receive_state *s)
 	    {
 	      memcpy(s->next_out, s->next_in, avail);
 	      rsync_update(s, avail);
+	      s->next_in += avail;
+	      s->avail_in -= avail;
 	      s->i -= avail;
 	    }
 	  else
 	    {
 	      memcpy(s->next_out, s->next_in, s->i);
 	      rsync_update(s, s->i);
+	      s->next_in += s->i;
+	      s->avail_in -= s->i;
 	      goto do_token;
 	    }
 	}
 	break;
 
       do_lookup:
-	s->state = STATE_LOOKUP;
+	s->state = RSYNC_READ_LOOKUP;
 	s->i = 0;
-      case STATE_LOOKUP:
+      case RSYNC_READ_LOOKUP:
 	{
 	  UINT32 done;
 
@@ -121,7 +114,7 @@ rsync_receive(struct rsync_receive_state *s)
 	    case -1:
 	      return RSYNC_INPUT_ERROR;
 	    default:
-	      assert(0);
+	      abort();
 	    }
 	}
 	break;
@@ -129,31 +122,31 @@ rsync_receive(struct rsync_receive_state *s)
       do_checksum:
 	/* i is number of octets read */
 	s->i = 0;
-	md5_final(&s->full_sum);
-	md5_digest(&s->full_sum, s->buf);
-	s->state = STATE_CHECKSUM;
-      case STATE_CHECKSUM:
+	md5_final(&s->sum_md5);
+	md5_digest(&s->sum_md5, MD5_DIGEST_SIZE, s->buf);
+	s->state = RSYNC_READ_CHECKSUM;
+      case RSYNC_READ_CHECKSUM:
 	if (!s->avail_in)
 	  return progress ? RSYNC_PROGRESS : RSYNC_BUF_ERROR;
 
 	if (GET() != s->buf[s->i++])
 	  return RSYNC_INPUT_ERROR;
 
-	if (s->i == MD5_DIGESTSIZE)
+	if (s->i == RSYNC_SUM_SIZE)
 	  {
-	    s->state = STATE_INVALID;
+	    s->state = RSYNC_READ_INVALID;
 	    return RSYNC_DONE;
 	  }
 	break;
 
       default:
-	assert(0);
+	abort();
       }
 }
 
 void
 rsync_receive_init(struct rsync_receive_state *s)
 {
-  s->state = STATE_TOKEN;
+  s->state = RSYNC_READ_TOKEN;
   s->i = 0;
 }
