@@ -1,6 +1,6 @@
 /* server.h
  *
- * $Id: server.h,v 1.18 1999/04/25 15:16:15 nisse Exp $ */
+ * $Id: server.h,v 1.19 2001/10/03 15:58:27 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -29,4 +29,9 @@
 
 struct command *make_offer_service(struct alist *services);
 
+int
+read_host_key(const char *file,
+              struct alist *signature_algorithms,
+              struct alist *keys);
+
 #endif /* LSH_SERVER_H_INCLUDED */
