@@ -1,6 +1,6 @@
 /* server_pty.c
  *
- * $Id: server_pty.c,v 1.20 2001/04/08 14:37:37 nisse Exp $
+ * $Id: server_pty.c,v 1.21 2001/09/01 10:43:38 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -77,7 +77,7 @@ make_pty_info(void)
 {
   NEW(pty_info, pty);
 
-  resource_init(&pty->super, do_kill_pty_info);
+  init_resource(&pty->super, do_kill_pty_info);
   pty->tty_name = NULL;
   pty->master = pty->slave = -1;
   return pty;
