@@ -1,6 +1,6 @@
 /* server_session.c
  *
- * $Id: server_session.c,v 1.66 2001/09/01 19:02:55 nisse Exp $
+ * $Id: server_session.c,v 1.73 2001/09/28 21:27:00 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -115,18 +115,17 @@ do_send_adjust(struct ssh_channel *s,
     {
       assert(session->out->read);
 
-      session->out->want_read = 1;
+      lsh_oop_register_read_fd(session->out);
     }
   
   if (session->err && session->err->super.alive)
     {
       assert(session->err->read);
   
-      session->err->want_read = 1;
+      lsh_oop_register_read_fd(session->err);
     }
 }
 
-/* FIXME: Perhaps unify with client_session.c:do_client_session_eof */
 static void
 do_eof(struct ssh_channel *channel)
 {
@@ -134,7 +133,7 @@ do_eof(struct ssh_channel *channel)
 
   trace("server_session.c: do_eof\n");
 
-  write_buffer_close(session->in->write_buffer);
+  close_fd_nicely(session->in);
 }
 
 struct ssh_channel *
@@ -296,13 +295,6 @@ make_exit_shell(struct server_session *session)
   return &self->super;
 }
 
-/* GABA:
-   (class
-     (name shell_request)
-     (super channel_request)
-     (vars
-       (backend object io_backend)))
-*/
 
 static int
 make_pipes(int *in, int *out, int *err)
@@ -336,18 +328,20 @@ make_pipes(int *in, int *out, int *err)
 #define BASH_WORKAROUND 1
 
 #if WITH_PTY_SUPPORT
-static int make_pty(struct pty_info *pty, int *in, int *out, int *err)
-{
-  int saved_errno = 0;
 
+static int
+make_pty(struct pty_info *pty, int *in, int *out, int *err)
+{
   debug("make_pty... ");
-  if (pty)
-    debug("exists: \n"
-	  "  alive = %i\n"
-	  "  master = %i\n"
-	  "  slave = %i\n"
-	  "... ",
-	  pty->super.alive, pty->master, pty->slave);
+
+  assert(pty);
+  
+  debug("exists: \n"
+        "  alive = %i\n"
+        "  master = %i\n"
+        "  slave = %i\n"
+        "... ",
+        pty->super.alive, pty->master, pty->slave);
   debug("\n");
   
   if (pty) 
@@ -355,52 +349,77 @@ static int make_pty(struct pty_info *pty, int *in, int *out, int *err)
       assert(pty->super.alive);
       
       debug("make_pty: Using allocated pty.\n");
+
+      /* Ownership of the master fd:s is passed on to some file
+       * objects. We need an fd for window_change_request, but we have
+       * to use one of our regular fd:s to the master side, or we're
+       * disrupt EOF handling on either side. */
+
+      pty->super.alive = 0;
+      
+      /* FIXME: It seems unnecessary to dup all the fd:s here. We
+       * could use a single lsh_fd object for the master side of the
+       * pty. */
+
       in[0] = pty->slave;
       in[1] = pty->master;
-
-      /* Ownership of the fd:s passes on to some file objects. */
-      pty->super.alive = 0;
-
-      /* FIXME: It seems unnecessary to dup the fd:s here. But perhaps
-       * having equal in and out fds may confuse the cleanup code, so
-       * we leave it for now. */
-      if ((out[0] = dup(pty->master)) != -1)
+          
+      if ((out[0] = dup(pty->master)) < 0)
         {
-          if ((out[1] = dup(pty->slave)) != -1) 
-            {
-#if BASH_WORKAROUND
-	      /* Don't use a separate stderr channel; just dup the
-	       * stdout pty to stderr. */
-	      if ((err[1] = dup(pty->slave)) != -1)
-                {
-                  err[0] = -1;
-                  return 1;
-                } 
-#else /* !BASH_WORKAROUND */
-	      if (lsh_make_pipe(err))
-		{
-		  /* Success! */
-		  return 1;
-		}
-#endif /* !BASH_WORKAROUND */
-              saved_errno = errno;
-	      
-            }
-	  else
-	    saved_errno = errno;
-	  close(out[0]);
-	}
-      else 
-	saved_errno = errno;
-      close(in[0]);
-      close(in[1]);
+          werror("make_pty: duping master pty to stdout failed (errno = %i): %z\n",
+                 errno, STRERROR(errno));
 
-      werror("make_pty: duping pty filedescriptors failed (errno = %i): %z\n",
-	     errno, STRERROR(errno));
+          close(in[0]);
+          close(in[1]);
+
+          return 0;
+        }
+
+      if ((out[1] = dup(pty->slave)) < 0)
+        {
+          werror("make_pty: duping slave pty to stdout failed (errno = %i): %z\n",
+                 errno, STRERROR(errno));
+
+          close(in[0]);
+          close(in[1]);
+          close(out[0]);
+
+          return 0;
+        }
+#if BASH_WORKAROUND
+      /* Don't use a separate stderr channel; just dup the
+       * stdout pty to stderr. */
+      if ((err[1] = dup(pty->slave)) < 0)
+        {
+          werror("make_pty: duping slave pty to stdout failed (errno = %i): %z\n",
+                 errno, STRERROR(errno));
+
+          close(in[0]);
+          close(in[1]);
+          close(out[0]);
+          close(out[1]);
+
+          return 0;
+        }
+      
+      err[0] = -1;
+      
+#else /* !BASH_WORKAROUND */
+      if (!lsh_make_pipe(err))
+        {
+          close(in[0]);
+          close(in[1]);
+          close(out[0]);
+          close(out[1]);
+
+          return 0;
+        }
+#endif /* !BASH_WORKAROUND */
+      return 1;
     }
-  errno = saved_errno;
   return 0;
 }
+
 #else /* !WITH_PTY_SUPPORT */
 static int make_pty(struct pty_info *pty UNUSED,
 		    int *in UNUSED, int *out UNUSED, int *err UNUSED)
@@ -411,15 +430,12 @@ static int make_pty(struct pty_info *pty UNUSED,
 static int
 spawn_process(struct server_session *session,
 	      struct lsh_user *user,
-	      struct address_info *peer,
-	      struct io_backend *backend)
+	      struct address_info *peer)
 {
   int in[2];
   int out[2];
   int err[2];
 
-  int using_pty = 0;
-  
   if (session->process)
     /* Already spawned a shell or command */
     return -1;
@@ -427,10 +443,13 @@ spawn_process(struct server_session *session,
   /* {in|out|err}[0] is for reading,
    * {in|out|err}[1] for writing. */
 
-  if (make_pty(session->pty, in, out, err))
-    using_pty = 1;
+  if (session->pty && !make_pty(session->pty, in, out, err))
+    {
+      KILL(session->pty);
+      session->pty = NULL;
+    }
 
-  else if (!make_pipes(in, out, err))
+  if (!session->pty && !make_pipes(in, out, err))
     return -1;
 
   {
@@ -438,7 +457,7 @@ spawn_process(struct server_session *session,
     
     if (USER_FORK(user, &child,
 		  make_exit_shell(session),
-		  peer, using_pty ? session->pty->tty_name : NULL))
+		  peer, session->pty ? session->pty->tty_name : NULL))
       {
 	if (child)
 	  { /* Parent */
@@ -465,7 +484,7 @@ spawn_process(struct server_session *session,
 		= make_channel_read_close_callback(channel);
 
 	      session->in
-		= io_write(make_lsh_fd(backend, in[1], "child stdin",
+		= io_write(make_lsh_fd(in[1], "child stdin",
 				       io_exception_handler),
 			   SSH_MAX_PACKET, NULL);
 	  
@@ -476,13 +495,13 @@ spawn_process(struct server_session *session,
 	       * which will close the channel on read errors, or is it
 	       * better to just send EOF on read errors? */
 	      session->out
-		= io_read(make_lsh_fd(backend, out[0], "child stdout",
+		= io_read(make_lsh_fd(out[0], "child stdout",
 				      io_exception_handler),
 			  make_channel_read_data(channel),
 			  read_close_callback);
 	      session->err 
 		= ( (err[0] != -1)
-		    ? io_read(make_lsh_fd(backend, err[0], "child stderr",
+		    ? io_read(make_lsh_fd(err[0], "child stderr",
 					  io_exception_handler),
 			      make_channel_read_stderr(channel),
 			      read_close_callback)
@@ -534,7 +553,7 @@ spawn_process(struct server_session *session,
 	      }
 	    
 #if WITH_PTY_SUPPORT
-	    if (using_pty)
+	    if (session->pty)
 	      {
 		debug("lshd: server.c: Setting controlling tty...\n");
 		if (!tty_setctty(session->pty))
@@ -549,10 +568,9 @@ spawn_process(struct server_session *session,
 	      }
 #endif /* WITH_PTY_SUPPORT */
 	  
-	    /* Close all descriptors but those used for
-	     * communicationg with parent. We rely on the
-	     * close-on-exec flag for all fd:s handled by the
-	     * backend. */
+	    /* Close all descriptors but those used for communicationg
+	     * with parent. We rely on the close-on-exec flag for all
+	     * other fd:s. */
 	    
 	    if (dup2(in[0], STDIN_FILENO) < 0)
 	      {
@@ -600,15 +618,14 @@ spawn_process(struct server_session *session,
   return -1;
 }
 
-static void
-do_spawn_shell(struct channel_request *s,
-	       struct ssh_channel *channel,
-	       struct channel_request_info *info UNUSED,
-	       struct simple_buffer *args,
-	       struct command_continuation *c,
-	       struct exception_handler *e)
+DEFINE_CHANNEL_REQUEST(shell_request_handler)
+     (struct channel_request *s UNUSED,
+      struct ssh_channel *channel,
+      struct channel_request_info *info UNUSED,
+      struct simple_buffer *args,
+      struct command_continuation *c,
+      struct exception_handler *e)
 {
-  CAST(shell_request, closure, s);
   CAST(server_session, session, channel);
 
   static struct exception shell_request_failed =
@@ -625,8 +642,7 @@ do_spawn_shell(struct channel_request *s,
     goto fail;
 
   switch (spawn_process(session, channel->connection->user,
-			channel->connection->peer,
-			closure->backend))
+			channel->connection->peer))
     {
     case 1: /* Parent */
       /* NOTE: The return value is not used. */
@@ -684,27 +700,14 @@ do_spawn_shell(struct channel_request *s,
   EXCEPTION_RAISE(e, &shell_request_failed);
 }
 
-struct channel_request *
-make_shell_handler(struct io_backend *backend)
+DEFINE_CHANNEL_REQUEST(exec_request_handler)
+     (struct channel_request *s UNUSED,
+      struct ssh_channel *channel,
+      struct channel_request_info *info UNUSED,
+      struct simple_buffer *args,
+      struct command_continuation *c,
+      struct exception_handler *e)
 {
-  NEW(shell_request, closure);
-
-  closure->super.handler = do_spawn_shell;
-  closure->backend = backend;
-  
-  return &closure->super;
-}
-
-
-static void
-do_spawn_exec(struct channel_request *s,
-	      struct ssh_channel *channel,
-	      struct channel_request_info *info UNUSED,
-	      struct simple_buffer *args,
-	      struct command_continuation *c,
-	      struct exception_handler *e)
-{
-  CAST(shell_request, closure, s);
   CAST(server_session, session, channel);
 
   static struct exception exec_request_failed =
@@ -731,8 +734,7 @@ do_spawn_exec(struct channel_request *s,
       struct lsh_string *command_line = ssh_format("%ls", command_len, command);
       
       switch (spawn_process(session, channel->connection->user,
-			    channel->connection->peer,
-			    closure->backend))
+			    channel->connection->peer))
 	{
 	case 1: /* Parent */
 	  lsh_string_free(command_line);
@@ -792,24 +794,13 @@ do_spawn_exec(struct channel_request *s,
     }
 }
 
-struct channel_request *
-make_exec_handler(struct io_backend *backend)
-{
-  NEW(shell_request, closure);
-
-  closure->super.handler = do_spawn_exec;
-  closure->backend = backend;
-  
-  return &closure->super;
-}
-
 /* For simplicity, represent a subsystem simply as a name of the
  * executable. */
 
 /* GABA:
    (class
      (name subsystem_request)
-     (super shell_request)
+     (super channel_request)
      (vars
        ;(subsystems object alist)
        ; A list { name, program, name, program, NULL }
@@ -878,8 +869,7 @@ do_spawn_subsystem(struct channel_request *s,
 	}
       
       switch (spawn_process(session, channel->connection->user,
-			    channel->connection->peer,
-			    self->super.backend))
+			    channel->connection->peer))
 	{
 	case 1: /* Parent */
 	  /* NOTE: The return value is not used. */
@@ -911,16 +901,14 @@ do_spawn_subsystem(struct channel_request *s,
 }
 
 struct channel_request *
-make_subsystem_handler(struct io_backend *backend,
-		       const char **subsystems)
+make_subsystem_handler(const char **subsystems)
 {
   NEW(subsystem_request, self);
 
-  self->super.super.handler = do_spawn_subsystem;
-  self->super.backend = backend;
+  self->super.handler = do_spawn_subsystem;
   self->subsystems = subsystems;
   
-  return &self->super.super;
+  return &self->super;
 }
 
 
@@ -955,8 +943,9 @@ do_alloc_pty(struct channel_request *c UNUSED,
       && parse_string(args, &mode_length, &mode)
       && parse_eod(args))
     {
-      /* The client may only request a tty once. */
-      if (session->pty)
+      /* The client may only request one tty, and only before
+       * starting a process. */
+      if (session->pty || session->process)
 	{
 	  lsh_string_free(term);
 	  EXCEPTION_RAISE(e, &pty_request_failed);
@@ -988,7 +977,7 @@ do_alloc_pty(struct channel_request *c UNUSED,
 		      tty_setwinsize(pty->slave, &dims))
 		    {
 		      REMEMBER_RESOURCE(channel->resources, &pty->super);
-		      
+
 		      verbose(" granted.\n");
 		      COMMAND_RETURN(s, NULL);
 		      
@@ -1040,22 +1029,15 @@ do_window_change_request(struct channel_request *c UNUSED,
       static const struct exception winch_request_failed =
 	STATIC_EXCEPTION(EXC_CHANNEL_REQUEST, "window-change request failed: No pty");
 
-      if (!(session->pty && session->pty->super.alive))
-	EXCEPTION_RAISE(e, &winch_request_failed);
-      
-      else if (!tty_setwinsize(session->pty->slave, &dims))
-	EXCEPTION_RAISE(e, &winch_request_failed);
-
+      if (session->pty && session->in && session->in->super.alive
+          && tty_setwinsize(session->in->fd, &dims))
+        /* Success. Rely on the terminal driver sending SIGWINCH */
+        COMMAND_RETURN(s, NULL);
       else
-	{
-	  if (!SIGNAL_PROCESS(session->process, SIGWINCH))
-	    werror("Sending SIGWINCH signal failed.\n");
-
-	  COMMAND_RETURN(s, NULL);
-	}
+        EXCEPTION_RAISE(e, &winch_request_failed);
     }
   else
-    PROTOCOL_ERROR(e, "Invalid window-change request.");
+    PROTOCOL_ERROR(channel->connection->e, "Invalid window-change request.");
 }
 
 struct channel_request
