@@ -1,6 +1,6 @@
 /* server_userauth.h
  *
- * $Id: server_userauth.h,v 1.18 2001/08/21 11:13:36 nisse Exp $
+ * $Id: server_userauth.h,v 1.19 2001/09/24 16:51:13 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -54,7 +54,7 @@
 #define USER_LOOKUP(db, n, f) ((db)->lookup((db), (n), (f)))
 
 struct user_db *
-make_unix_user_db(struct io_backend *backend, struct reap *reaper,
+make_unix_user_db(struct reap *reaper,
 		  const char *pw_helper, const char *login_shell,
 		  int allow_root);
 
