@@ -2,7 +2,7 @@
  *
  * Reads a sexp in given form from, and writes it in given form.
  *
- * $Id: sexp-conv.c,v 1.8 2001/07/02 21:09:48 nisse Exp $ */
+ * $Id: sexp-conv.c,v 1.11 2001/10/02 20:56:03 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -27,11 +27,12 @@
 #include "alist.h"
 #include "atoms.h"
 #include "crypto.h"
+#include "format.h"
 #include "io.h"
 #include "lsh.h"
 #include "lsh_argp.h"
-#include "sexp_commands.h"
-#include "spki_commands.h"
+#include "sexp.h"
+#include "spki.h"
 #include "version.h"
 #include "werror.h"
 #include "xalloc.h"
@@ -42,138 +43,75 @@
 
 #include "sexp-conv.c.x"
 
-/* Global, for simplicity */
-int exit_code = EXIT_SUCCESS;
-
-/* GABA:
-   (expr
-     (name make_sexp_conv)
-     (params
-       (read object command)
-       (select object command)
-       (transform object command)
-       (print object command)
-       (dest object abstract_write))
-     (expr
-       (lambda (in)
-         (print dest (transform (select (read in)))))))
-*/
-
-
-static void
-do_exc_sexp_conv_handler(struct exception_handler *self,
-			 const struct exception *x)
-{
-  /* CAST(exc_sexp_conv_handler, self, s); */
-  
-  switch (x->type)
-    {
-    case EXC_SEXP_SYNTAX:
-      werror("Invalid SEXP input.\n");
-      exit_code = EXIT_FAILURE;
-      /* Fall through */
-    case EXC_SEXP_EOF:
-      /* Normal termination */
-      EXCEPTION_RAISE(self->parent, &finish_read_exception);
-      break;
-    case EXC_IO_WRITE:
-    case EXC_IO_READ:
-      {
-	CAST(io_exception, e, x);
-	exit_code = EXIT_FAILURE;
-	werror("sexp-conv: %z, (errno = %i)\n", x->msg, e->error);
-	break;
-      }
-    default:
-      exit_code = EXIT_FAILURE;
-      EXCEPTION_RAISE(self->parent, x);
-      return;
-    }
-}
 
 enum sexp_select_ops { OP_CAR, OP_CDR, OP_RETURN_CAR };
 
-/* GABA:
-   (class
-     (name sexp_select_command)
-     (super command)
-     (vars
-       (ops object int_list)))
-*/
-
-static void
-do_sexp_select(struct command *s,
-	       struct lsh_object *x,
-	       struct command_continuation *c,
-	       struct exception_handler *e)
+static struct sexp *
+process_select(struct int_list *select,
+	       struct sexp *expr)
 {
-  CAST(sexp_select_command, self, s);
-  CAST_SUBTYPE(sexp, expr, x);
   struct sexp_iterator *i;
   unsigned j;
 
-  static const struct exception non_list =
-    STATIC_EXCEPTION(EXC_SEXP_SYNTAX,
-		     "Attemt to select in a non-list expression.");
-
-  static const struct exception list_exhausted =
-    STATIC_EXCEPTION(EXC_SEXP_SYNTAX,
-		     "List exhausted while selecting.");
-
   if (sexp_atomp(expr))
     {
-      EXCEPTION_RAISE(e, &non_list);
-      return;
+      werror("Attempt select in a non-list expression.\n");
+      return NULL;
     }
 
   i = SEXP_ITER(expr);
-
-  for (j = 0; j < LIST_LENGTH(self->ops); j++)
-    switch (LIST(self->ops)[j])
+  assert(i);
+  
+  for (j = 0; j < LIST_LENGTH(select); j++)
+    switch (LIST(select)[j])
       {
       case OP_RETURN_CAR:
-	{
-	  struct sexp *expr = SEXP_GET(i);
-	  if (!expr)
-	    EXCEPTION_RAISE(e, &list_exhausted);
-	  else
-	    COMMAND_RETURN(c, expr);
+	expr = SEXP_GET(i);
+	if (!expr)
+	  {
+	    werror("List exhausted while selecting.\n");
+	    return NULL;
+	  }
+	return expr;
+	
+	break;
 
-	  return;
-	}
       case OP_CAR:
-	{
-	  struct sexp *expr = SEXP_GET(i);
-	  if (!expr)
-	    {
-	      EXCEPTION_RAISE(e, &list_exhausted);
-	      return;
-	    }
-	    
-	  if (sexp_atomp(expr))
-	    {
-	      EXCEPTION_RAISE(e, &non_list);
-	      return;
-	    }
-	  i = SEXP_ITER(expr);
+	expr = SEXP_GET(i);
+	if (!expr)
+	  {
+	    werror("List exhausted while selecting.\n");
+	    return NULL;
+	  }
 
-	  break;
-	}
+	if (sexp_atomp(expr))
+	  {
+	    werror("Attempt select in a non-list expression.\n");
+	    return NULL;
+	  }
+	  
+	i = SEXP_ITER(expr);
+	assert(i);
+	  
+	break;
+	  
       case OP_CDR:
 	SEXP_NEXT(i);
+
 	if (!SEXP_LEFT(i))
 	  {
-	    EXCEPTION_RAISE(e, &list_exhausted);
-	    return;
+	    werror("List exhausted while selecting.\n");
+
+	    return NULL;
 	  }
 	break;
       }
   
-  fatal("do_sexp_select: Internal error!\n");
+  fatal("process_select: Internal error!\n");
 }
-     
-static struct command *
-make_sexp_select_command(const char *arg)
+
+static struct int_list *
+parse_select(const char *arg)
 {
   unsigned len = strlen(arg);
 
@@ -202,17 +140,10 @@ make_sexp_select_command(const char *arg)
 
       LIST(ops)[i++] = OP_RETURN_CAR;
       assert (i == LIST_LENGTH(ops));
-      
-      {
-	NEW(sexp_select_command, self);
-	self->super.call = do_sexp_select;
-	self->ops = ops;
 
-	return &self->super;
-      }
+      return ops;
     }
-  else
-    return NULL;
+  return NULL;
 }
 
 /* Option parsing */
@@ -255,28 +186,28 @@ main_options[] =
     (once . int)
     (mode . int)
     (algorithms object alist)
-    (hash . int)
-    (select object command)
-    (transform object command)
-    (print object command)
-))
+    (hash object hash_algorithm)
+    (hash_name . int)
+    (select object int_list)))
 */
 
-static struct sexp_conv_options *make_options(void)
+static struct sexp_conv_options *
+make_options(void)
 {
   NEW(sexp_conv_options, self);
   self->input = SEXP_TRANSPORT;
   self->output = SEXP_ADVANCED;
   self->once = 0;
   self->mode = MODE_VANILLA;
-  self->select = &command_I;
-  self->transform = &command_I;
+  self->select = NULL;
+  
   self->algorithms = make_alist(2,
 				ATOM_MD5, &md5_algorithm,
 				ATOM_SHA1, &sha1_algorithm,
 				-1);
-  self->hash = ATOM_SHA1;
-
+  self->hash = NULL;
+  self->hash_name = ATOM_SHA1;
+  
   return self;
 }
 
@@ -305,42 +236,21 @@ main_argp_parser(int key, char *arg, struct argp_state *state)
       break;
     case ARGP_KEY_END:
       {
-	switch(self->mode)
-	  {
-	  default:
-	    fatal("Internal error!");
-	  case MODE_VANILLA:
-	    self->transform = &command_I;
-	    self->print = &make_sexp_print_command(self->output)->super;
-	    break;
-	  case MODE_SPKI_HASH:
-	    {
-	      CAST_SUBTYPE(hash_algorithm, a,
-			   ALIST_GET(self->algorithms, self->hash));
-	      self->transform = make_spki_hash(self->hash, a);
-	      self->print = &make_sexp_print_command(self->output)->super;
-	      break;
-	    }
-	  case MODE_RAW_HASH:
-	    {
-	      CAST_SUBTYPE(hash_algorithm, a,
-			   ALIST_GET(self->algorithms, self->hash));
-	      self->transform = &command_I;
-	      self->print = make_sexp_print_raw_hash(a);
-	      break;
-	    }
-	  }
+	CAST_SUBTYPE(hash_algorithm, h,
+		     ALIST_GET(self->algorithms, self->hash_name));
+
+	assert(h);
+
+	self->hash = h;
 	break;
       }
     case OPT_HASH:
-      {
-	int hash = lookup_hash(self->algorithms, arg, NULL, 0);
-	if (hash)
-	  self->hash = hash;
-	else
-	  argp_error(state, "Unknown hash algorithm '%s'.", arg);
-	break;
-      }
+      self->hash_name = lookup_hash(self->algorithms, arg, NULL, 0);
+      
+      if (!self->hash_name)
+	argp_error(state, "Unknown hash algorithm '%s'.", arg);
+      break;
+
     case OPT_SPKI_HASH:
       self->mode = MODE_SPKI_HASH;
       break;
@@ -348,15 +258,13 @@ main_argp_parser(int key, char *arg, struct argp_state *state)
       self->mode = MODE_RAW_HASH;
       break;
     case OPT_SELECT:
-      {
-	struct command *sel = make_sexp_select_command(arg);
-	if (sel)
-	  self->select = sel;
-	else
-	  argp_error(state, "Unsupported select operator '%s' (the supported "
-		     "ones are ca[ad]*r).", arg);
-	break;
-      }
+      self->select = parse_select(arg);
+      
+      if (!self->select)
+	argp_error(state, "Unsupported select operator '%s' (the supported "
+		   "ones are ca[ad]*r).", arg);
+      break;
+
     case OPT_ONCE:
       self->once = 1;
       break;
@@ -384,43 +292,89 @@ main_argp =
 int main(int argc, char **argv)
 {
   struct sexp_conv_options *options = make_options();
-  struct exception_handler *e;
-  struct io_backend *backend = make_io_backend();
+  const struct exception *e;
+  struct lsh_string *input;
+  struct lsh_string *output;
+  struct simple_buffer buffer;
   
   argp_parse(&main_argp, argc, argv, 0, NULL, options);
 
-  /* Patch the parent pointer later */
-  e = make_exception_handler(do_exc_sexp_conv_handler,
-			     NULL, HANDLER_CONTEXT);
+  input = io_read_file_raw(STDIN_FILENO, 1000);
+  if (!input)
+    {
+      werror("Failed to read stdin.\n");
+      return EXIT_FAILURE;
+    }
+
+  simple_buffer_init(&buffer, input->length, input->data);
+
+  while (!parse_eod(&buffer))
+    {
+      struct sexp *expr = sexp_parse(options->input, &buffer);
+
+      if (!expr)
+	{
+	  werror("S-expression syntax error.\n");
+	  return EXIT_FAILURE;
+	}
+
+      if (options->select)
+	{
+	  expr = process_select(options->select, expr);
+	  if (!expr)
+	    return EXIT_FAILURE;
+	}
+
+      /* FIXME: Perhaps move the addition of \n to sexp_format? */
+      switch (options->mode)
+	{
+	case MODE_VANILLA:
+	  output = sexp_format(expr, options->output, 0);
+
+	  if (options->output != SEXP_CANONICAL)
+	    output = ssh_format("%lfS\n", output);
+	  break;
+	  
+	case MODE_SPKI_HASH:
+	  output = sexp_format(spki_hash_sexp(options->hash,
+					      options->hash_name,
+					      expr),
+			       options->output,
+			       0);
+
+	  if (options->output != SEXP_CANONICAL)
+	    output = ssh_format("%lfS\n", output);
+	  
+	  break;
+	case MODE_RAW_HASH:
+	  output = ssh_format("%lxfS",
+			      hash_string(options->hash,
+					  sexp_format(expr, SEXP_CANONICAL, 0),
+					  1));
+	  break;
+
+	default:
+	  fatal("Internal error.\n");
+	}
+
+      e = write_raw(STDOUT_FILENO, output->length, output->data);
+      if (e)
+	{
+	  werror("%z\n", e->msg);
+	  return EXIT_FAILURE;
+	}
+      
+      lsh_string_free(output);
+
+      /* FIXME: Maybe gc here? */
+
+      if (options->once)
+	break;
+    }
+
+  lsh_string_free(input);
   
-  {
-    CAST_SUBTYPE(command, work,
-		 make_sexp_conv(
-		   make_read_sexp_command(options->input,
-					  !options->once,
-					  MAX_SEXP_SIZE),
-		   options->select,
-		   options->transform,
-		   options->print,
-		   &(io_write(make_lsh_fd(backend,
-					  STDOUT_FILENO,
-					  "stdout",
-					 e),
-			      SEXP_BUFFER_SIZE,
-			      NULL)
-		     ->write_buffer->super)));
-
-    struct lsh_fd *in = make_lsh_fd(backend, STDIN_FILENO, "stdin", e);
-
-    /* Fixing the exception handler creates a circularity */
-    e->parent = make_exc_finish_read_handler(in,
-					     &default_exception_handler,
-					     HANDLER_CONTEXT);
-    
-    COMMAND_CALL(work, in,
-		 &discard_continuation, e);
-  }
-  io_run(backend);
-
-  return exit_code;
+  gc_final();
+  
+  return EXIT_SUCCESS;
 }
