@@ -2,7 +2,7 @@
  *
  * An implementation of SPKI certificate checking
  *
- * $Id: spki.c,v 1.29 2001/05/01 18:35:57 nisse Exp $ */
+ * $Id: spki.c,v 1.34 2001/10/03 15:11:49 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -29,9 +29,11 @@
 #include "crypto.h"
 #include "format.h"
 #include "io.h"
+#include "interact.h"
 #include "list.h"
 #include "parse.h"
 #include "publickey_crypto.h"
+#include "randomness.h"
 #include "sexp.h"
 #include "werror.h"
 #include "xalloc.h"
@@ -134,7 +136,7 @@ spki_make_verifier(struct alist *algorithms,
   return v;
 }
 
-
+/* FIXME: Merge spki_make_signer and spki_sexp_to_signer? */
 /* Returns the algorithm type, or zero on error. */
 struct signer *
 spki_make_signer(struct alist *algorithms,
@@ -179,6 +181,40 @@ spki_make_signer(struct alist *algorithms,
   return s;
 }
 
+/* Reading keys */
+
+struct signer *
+spki_sexp_to_signer(struct alist *algorithms,
+                    struct sexp *key,
+                    int *algorithm_name)
+{
+  struct sexp_iterator *i;
+  
+  if ((i = sexp_check_type(key, ATOM_PRIVATE_KEY)))
+    {
+      struct sexp *expr = SEXP_GET(i);
+      struct signer *s;
+      
+      if (!expr)
+        {
+          werror("sexp_to_signer: Invalid key\n");
+          return NULL;
+        }
+      
+      s = spki_make_signer(algorithms, expr, algorithm_name);
+      
+      if (s)
+	/* Test key here? */
+	;
+      else
+	werror("sexp_to_signer: Invalid key.", expr);
+
+      return s;
+    }
+  werror("sexp_to_signer: Expected private-key expression.");
+  return NULL;
+}
+
 struct sexp *
 spki_make_public_key(struct verifier *verifier)
 {
@@ -227,6 +263,23 @@ spki_hash_data(struct hash_algorithm *algorithm,
 		sexp_s(NULL, out), -1);
 }  
 
+/* Create an SPKI hash from an s-expression. */
+
+struct sexp *
+spki_hash_sexp(struct hash_algorithm *algorithm,
+	       int name,
+	       struct sexp *expr)
+{
+  struct lsh_string *s = hash_string(algorithm,
+				     sexp_format(expr, SEXP_CANONICAL, 0),
+				     1);
+  struct sexp *hash = spki_hash_data(algorithm, name, 
+				     s->length, s->data);
+  lsh_string_free(s);
+
+  return hash;
+}
+
 
 /* 5-tuples */
 
@@ -548,7 +601,7 @@ spki_sexp_to_tag(struct sexp *e,
 	  return NULL;
 	}
       
-      if (sexp_check_type(e, ATOM_STAR, &i))
+      if ((i = sexp_check_type(e, ATOM_STAR)))
 	{
 	  struct sexp *magic = SEXP_GET(i);
 	  
@@ -621,7 +674,7 @@ spki_acl_entry_to_5_tuple(struct spki_context *ctx,
   if (!e)
     return NULL;
 
-  if (sexp_check_type(e, ATOM_PROPAGATE, &j))
+  if ((j = sexp_check_type(e, ATOM_PROPAGATE)))
     {
       if (SEXP_GET(j))
 	{
@@ -633,7 +686,7 @@ spki_acl_entry_to_5_tuple(struct spki_context *ctx,
       e = SEXP_GET(i);
     }
 
-  if (sexp_check_type(e, ATOM_TAG, &j))
+  if ((j = sexp_check_type(e, ATOM_TAG)))
     {
       struct sexp *tag = SEXP_GET(j);
       SEXP_NEXT(j);
@@ -667,7 +720,7 @@ spki_acl_entry_to_5_tuple(struct spki_context *ctx,
 
 /* Takes an spki_context and an ACL s-expression, and adds
  * corresponding 5-tuples to the context. Returns 1 if all entries
- * were correct, 0 on any error. However, it tries to gon on if some
+ * were correct, 0 on any error. However, it tries to go on if some
  * sub-expression is invalid. */
 
 int
@@ -677,23 +730,25 @@ spki_add_acl(struct spki_context *ctx,
   struct sexp_iterator *i;
   int res = 1;
   
-  if (!sexp_check_type(e, ATOM_ACL, &i))
+  if (!(i = sexp_check_type(e, ATOM_ACL)))
     {
       werror("spki_read_acls: Invalid acl\n");
       return 0;
     }
 
   /* FIXME: Accept at least (version "0") */
+#if 0
   if (sexp_check_type(SEXP_GET(i), ATOM_VERSION, NULL))
     {
       werror("spki_read_acls: Unsupported acl version\n");
       return 0;
     }
+#endif
   
   for (; (e = SEXP_GET(i)); SEXP_NEXT(i))
     {
       struct sexp_iterator *j;
-      if (sexp_check_type(e, ATOM_ENTRY, &j))
+      if ((j = sexp_check_type(e, ATOM_ENTRY)))
 	{
 	  struct spki_5_tuple *acl = spki_acl_entry_to_5_tuple(ctx, j);
 	  if (acl)
@@ -903,3 +958,270 @@ make_spki_context(struct alist *algorithms)
 
   return &self->super;
 }
+
+
+/* PKCS-5 handling */
+
+/* Encryption of private data.
+ * For PKCS#5 (version 2) key derivation, we use
+ *
+ * (password-encrypted LABEL
+ *   (Xpkcs5v2 hmac-sha1 (salt #...#)
+ *                       (iterations #...#))
+ *   ("3des-cbc" (iv #...#) (data #...#)))
+ *
+ * where the X:s will be removed when the format is more stable.
+ *
+ */
+
+struct sexp *
+spki_pkcs5_encrypt(struct randomness *r,
+                   struct lsh_string *label,
+		   UINT32 prf_name,
+		   struct mac_algorithm *prf,
+		   int crypto_name,
+		   struct crypto_algorithm *crypto,
+		   UINT32 salt_length,
+		   struct lsh_string *password,
+		   UINT32 iterations,
+                   struct lsh_string *data)
+{
+  struct lsh_string *key;
+  struct lsh_string *salt;
+  struct lsh_string *iv;
+  struct sexp *method;
+  
+  assert(crypto);
+  assert(prf);
+
+  salt = lsh_string_alloc(salt_length);
+  RANDOM(r, salt->length, salt->data);
+    
+  key = lsh_string_alloc(crypto->key_size);
+
+  pkcs5_derive_key(prf,
+		   password->length, password->data,
+		   salt->length, salt->data,
+		   iterations,
+		   key->length, key->data);
+
+  method = sexp_l(4, SA(XPKCS5V2), sexp_a(prf_name),
+                  sexp_l(2, SA(ITERATIONS), sexp_uint32(iterations), -1),
+                  sexp_l(2, SA(SALT), sexp_s(NULL, salt), -1), -1);
+
+  if (crypto->iv_size)
+    {
+      iv = lsh_string_alloc(crypto->iv_size);
+      RANDOM(r, iv->length, iv->data);
+    }
+  else
+    iv = NULL;
+
+  return sexp_l(4,
+                SA(PASSWORD_ENCRYPTED),
+                sexp_s(NULL, lsh_string_dup(label)),
+                method,
+                sexp_l(3,
+                       sexp_a(crypto_name),
+                       sexp_l(2, SA(IV), sexp_s(NULL, iv), -1),
+                       sexp_l(2, SA(DATA),
+                              sexp_s(NULL, crypt_string_pad
+                                     (MAKE_ENCRYPT(crypto,
+                                                   key->data,
+                                                   iv ? iv->data : NULL),
+                                      data, 0)),
+                              -1),
+                       -1),
+                -1);
+}
+
+struct sexp *
+spki_pkcs5_decrypt(struct alist *mac_algorithms,
+                   struct alist *crypto_algorithms,
+                   struct interact *interact,
+                   struct sexp *expr)
+{
+  struct sexp_iterator *i;
+  
+  if (!(i = sexp_check_type(expr, ATOM_PASSWORD_ENCRYPTED)))
+    return expr;
+
+  else
+    {
+      const struct lsh_string *label;
+      struct sexp *key_info;
+      struct sexp *payload;
+
+      struct crypto_algorithm *crypto;
+      struct mac_algorithm *mac;
+      
+      const struct lsh_string *salt;
+      const struct lsh_string *iv;
+      const struct lsh_string *data;
+      UINT32 iterations;
+      
+      if (SEXP_LEFT(i) != 3)
+	{
+	  werror("Invalid (password-encrypted ...) expression.\n");
+	  return NULL;
+	}
+	
+      /* NOTE: This is a place where it might make sense to use a sexp
+       * display type, but we don't support that for now. */
+      label = sexp2string(SEXP_GET(i));
+
+      if (!label)
+	{
+	  werror("Invalid label in (password-encrypted ...) expression.\n");
+	  return NULL;
+	}
+
+      SEXP_NEXT(i);
+      key_info = SEXP_GET(i);
+      assert(key_info);
+
+      SEXP_NEXT(i);
+      payload = SEXP_GET(i);
+      assert(payload);
+
+      /* Examine the payload expression first, before asking for a
+       * pass phrase. */
+
+      {
+	int algorithm_name = spki_get_type(payload, &i);
+	CAST_SUBTYPE(crypto_algorithm, tmp,
+		     ALIST_GET(crypto_algorithms, algorithm_name));
+	crypto = tmp;
+      }
+
+      if (!crypto)
+	{
+          werror("Unknown encryption algorithm for pkcs5v2.\n");
+	  return NULL;
+	}
+
+      iv = sexp2string(sexp_assq(i, ATOM_IV));
+
+      if (crypto->iv_size)
+	{
+	  if (!iv || (iv->length != crypto->iv_size))
+	    {
+	      werror("Invalid IV for pkcs5v2.\n");
+	      return NULL;
+	    }
+	}
+      else if (iv)
+	{
+	  if (iv->length)
+	    {
+	      werror("Unexpected iv provided for pkcs5v2.\n");
+	      return NULL;
+	    }
+	  iv = NULL;
+	}
+	
+      data = sexp2string(sexp_assq(i, ATOM_DATA));
+
+      if (!data)
+	{
+	  werror("Payload data missing for pkcs5v2.\n");
+	  return NULL;
+	}
+
+      if (crypto->block_size && (data->length & crypto->block_size))
+	{
+	  werror("Payload data doesn't match block size for pkcs5v2.\n");
+	  return NULL;
+	}
+
+      /* Get key */
+      switch (spki_get_type(key_info, &i)) 
+	{
+	default:
+	  werror("Unknown key derivation mechanism.\n");
+	  return NULL;
+
+	case ATOM_XPKCS5V2:
+	  if (SEXP_LEFT(i) != 3)
+	    {
+	      werror("Invalid pkcs5v2 parameters.\n");
+	      return NULL;
+	    }
+	  
+	  {
+	    int algorithm_name = sexp2atom(SEXP_GET(i));
+	    
+	    CAST_SUBTYPE(mac_algorithm, tmp,
+			 ALIST_GET(mac_algorithms,
+				   algorithm_name));
+
+	    mac = tmp;
+	  }
+
+	  if (!mac)
+	    {
+	      werror("Unknown mac for pkcs5v2.\n");
+	      return NULL;
+	    }
+
+	  SEXP_NEXT(i);
+	  if (!sexp2uint32(sexp_assq(i, ATOM_ITERATIONS), &iterations)
+	      || !iterations)
+	    {
+	      werror("Invalid iteration count for pkcs5v2.\n");
+	      return NULL;
+	    }
+	    
+	  salt = sexp2string(sexp_assq(i, ATOM_SALT));
+
+	  if (!salt)
+	    {
+	      werror("Invalid salt for pkcs5v2.\n");
+	      return NULL;
+	    }
+
+	  /* Do the work */
+
+	  {
+	    struct lsh_string *password
+	      = INTERACT_READ_PASSWORD(interact, 500, label, 0);
+	    struct lsh_string *clear;
+	    struct sexp *res;
+	    UINT8 *key;
+	    
+	    if (!password)
+	      {
+		werror("No password provided for pkcs5v2.");
+		return NULL;
+	      }
+
+	    key = alloca(crypto->key_size);
+	    pkcs5_derive_key(mac,
+			     password->length, password->data,
+			     salt->length, salt->data,
+			     iterations,
+			     crypto->key_size, key);
+
+	    lsh_string_free(password);
+
+	    clear = crypt_string_unpad(MAKE_DECRYPT(crypto,
+						    key,
+						    iv ? iv->data : NULL),
+				       data, 0);
+
+	    if (!clear)
+	      {
+		werror("Bad password for pkcs5v2.\n");
+		return NULL;
+	      }
+
+	    res = string_to_sexp(SEXP_CANONICAL, clear, 1);
+
+	    if (!res)
+              werror("Bad password for pkcs5v2.\n");
+
+            return res;
+	  }
+	}
+    }
+}
