@@ -2,7 +2,7 @@
  *
  * An implementation of SPKI certificate checking
  *
- * $Id: spki.h,v 1.19 2000/10/03 21:50:18 nisse Exp $ */
+ * $Id: spki.h,v 1.24 2001/10/03 15:12:01 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -62,8 +62,6 @@ make_spki_exception(UINT32 type, const char *msg, struct sexp *expr);
 
 int spki_get_type(struct sexp *e, struct sexp_iterator **res);
 
-int spki_check_type(struct sexp *e, int type, struct sexp_iterator **res);
-
 struct sexp *
 make_ssh_hostkey_tag(struct address_info *host);
 
@@ -76,32 +74,23 @@ spki_make_signer(struct alist *algorithms,
 		 struct sexp *e,
 		 int *type);
 
-struct sexp *
-spki_make_public_key(struct signer *signer);
+struct signer *
+spki_sexp_to_signer(struct alist *algorithms,
+                    struct sexp *key,
+                    int *algorithm_name);
 
 struct sexp *
-dsa_to_spki_public_key(struct dsa_public *p);
-
-extern struct command_simple spki_signer2public;
-#define SIGNER2PUBLIC (&spki_signer2public.super.super)
-
-extern struct command_simple spki_sexp2signer_command;
-#define SEXP2SIGNER (&spki_sexp2signer_command.super.super)
-
-extern struct command_simple spki_sexp2keypair_command;
-#define SEXP2KEYPAIR (&spki_sexp2keypair_command.super.super)
+spki_make_public_key(struct verifier *verifier);
 
 struct sexp *
 spki_hash_data(struct hash_algorithm *algorithm,
 	       int algorithm_name,
 	       UINT32 length, UINT8 *data);
 
-struct command *
-make_spki_hash(int name, struct hash_algorithm *algorithm);
-
-struct command *
-make_spki_parse_key(struct alist *algorithms);
-
+struct sexp *
+spki_hash_sexp(struct hash_algorithm *algorithm,
+	       int name,
+	       struct sexp *expr);
 
 /* At a point in time, not all fields are known; fields may be added
  * later, or computed as needed. This information is not automatically
@@ -230,4 +219,23 @@ int
 spki_add_acl(struct spki_context *ctx,
 	     struct sexp *e);
 
+
+struct sexp *
+spki_pkcs5_encrypt(struct randomness *r,
+                   struct lsh_string *label,
+		   UINT32 prf_name,
+		   struct mac_algorithm *prf,
+		   int crypto_name,
+		   struct crypto_algorithm *crypto,
+		   UINT32 salt_length,
+		   struct lsh_string *password,
+		   UINT32 iterations,
+                   struct lsh_string *data);
+
+struct sexp *
+spki_pkcs5_decrypt(struct alist *mac_algorithms,
+                   struct alist *crypto_algorithms,
+                   struct interact *interact,
+                   struct sexp *expr);
+
 #endif /* LSH_SPKI_H_INCLUDED */
