@@ -2,7 +2,7 @@
  *
  * Thomas Wu's Secure Remote Password Protocol
  *
- * $Id: srp_exchange.c,v 1.11 2001/08/12 20:27:46 nisse Exp $ */
+ * $Id: srp_exchange.c,v 1.12 2001/09/27 12:17:06 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -54,7 +54,7 @@ make_srp_entry(struct lsh_string *name, struct sexp *e)
 {
   struct sexp_iterator *i;
 
-  if (sexp_check_type(e, ATOM_SRP_VERIFIER, &i)
+  if ((i = sexp_check_type(e, ATOM_SRP_VERIFIER))
       && (SEXP_LEFT(i) == 3)
       && sexp_atom_eq(SEXP_GET(i), ATOM_SSH_RING1) )
     {
