@@ -2,7 +2,7 @@
  *
  * Interact with the user.
  *
- * $Id: unix_interact.c,v 1.9 2001/09/01 10:43:39 nisse Exp $ */
+ * $Id: unix_interact.c,v 1.13 2001/09/27 12:18:07 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -49,16 +49,6 @@
 
 #include "unix_interact.c.x"
 
-static volatile sig_atomic_t window_changed;
-
-static void winch_handler(int signum)
-{
-  assert(signum == SIGWINCH);
-
-  window_changed = 1;
-}
-
-
 /* Depends on the tty being line buffered */
 static int
 read_line(int fd, UINT32 size, UINT8 *buffer)
@@ -133,16 +123,41 @@ read_line(int fd, UINT32 size, UINT8 *buffer)
      (name window_subscriber)
      (super resource)
      (vars
+       (interact object unix_interact)
        (next object window_subscriber)
        (callback object window_change_callback)))
 */
 
+static void
+do_kill_window_subscriber(struct resource *s)
+{
+  CAST(window_subscriber, self, s);
+
+  if (self->super.alive)
+    {
+      self->super.alive = 0;
+      assert(self->interact->nsubscribers);
+      assert(self->interact->winch_handler);
+      
+      if (!--self->interact->nsubscribers)
+        {
+          KILL_RESOURCE(self->interact->winch_handler);
+          self->interact->winch_handler = NULL;
+        }
+    }
+}
+
+/* FIXME: Remove subscribers list, and register multiple liboop signal
+ * handlers instead? */
 /* GABA:
    (class
      (name unix_interact)
      (super interact)
      (vars
        (tty_fd . int)
+       ; Signal handler
+       (winch_handler object resource)
+       (nsubscribers . unsigned)
        (subscribers object window_subscriber)))
 */
 
@@ -297,22 +312,6 @@ unix_window_size(struct interact *s,
     && tty_getwinsize(self->tty_fd, d);
 }
 
-static struct resource *
-unix_window_change_subscribe(struct interact *s,
-			     struct window_change_callback *callback)
-{
-  CAST(unix_interact, self, s);
-
-  NEW(window_subscriber, subscriber);
-  init_resource(&subscriber->super, NULL);
-
-  subscriber->next = self->subscribers;
-  subscriber->callback = callback;
-
-  self->subscribers = subscriber;
-
-  return &subscriber->super;
-}
 
 /* GABA:
    (class
@@ -327,13 +326,16 @@ do_winch_handler(struct lsh_callback *s)
 {
   CAST(winch_handler, self, s);
   struct unix_interact *i = self->interact;
+
+  assert(!!i->nsubscribers == !!i->winch_handler);
   
   if (i->subscribers)
     {
       struct window_subscriber *s;
       struct window_subscriber **s_p;
+      unsigned alive;
 
-      for (s_p = &i->subscribers; (s = *s_p) ;)
+      for (alive = 0, s_p = &i->subscribers; (s = *s_p) ;)
 	{
 	  if (!s->super.alive)
 	    *s_p = s->next;
@@ -341,8 +343,11 @@ do_winch_handler(struct lsh_callback *s)
 	    {
 	      WINDOW_CHANGE_CALLBACK(s->callback, &i->super);
 	      s_p = &s->next;
+              alive++;
 	    }
 	}
+
+      assert(alive == i->nsubscribers);
     }
 }
 
@@ -356,8 +361,36 @@ make_winch_handler(struct unix_interact *i)
   return &self->super;
 }
 
+static struct resource *
+unix_window_change_subscribe(struct interact *s,
+			     struct window_change_callback *callback)
+{
+  CAST(unix_interact, self, s);
+
+  NEW(window_subscriber, subscriber);
+
+  init_resource(&subscriber->super, do_kill_window_subscriber);
+
+  subscriber->interact = self;
+  subscriber->next = self->subscribers;
+  subscriber->callback = callback;
+
+  self->subscribers = subscriber;
+  self->nsubscribers++;
+  
+  if (!self->winch_handler)
+    {
+      /* This is the first subscriber */
+      self->winch_handler
+        = io_signal_handler(SIGWINCH,
+                            make_winch_handler(self));
+    }
+  
+  return &subscriber->super;
+}
+
 struct interact *
-make_unix_interact(struct io_backend *backend)
+make_unix_interact(void)
 {
   NEW(unix_interact, self);
   
@@ -368,8 +401,6 @@ make_unix_interact(struct io_backend *backend)
   self->super.set_attributes = unix_set_attributes;
   self->super.window_size = unix_window_size;
   self->super.window_change_subscribe = unix_window_change_subscribe;
-  
-  window_changed = 0;
 
   self->tty_fd = -1;
   
@@ -380,24 +411,9 @@ make_unix_interact(struct io_backend *backend)
   self->tty_fd = open("/dev/tty", O_RDWR);
 #endif
 
-  if (backend && (self->tty_fd >= 0))
-    {
-      /* Track window changes. */
-      struct sigaction winch;
+  if (self->tty_fd >= 0)
+    /* Restore and reset tty if process is suspended. */      
+    suspend_handle_tty(self->tty_fd);
 
-      memset(&winch, 0, sizeof(winch));
-      winch.sa_handler = winch_handler;
-      sigemptyset(&winch.sa_mask);
-      winch.sa_flags = 0;
-
-      if (sigaction(SIGWINCH, &winch, NULL) < 0)
-	werror("Failed to install SIGWINCH handler (errno = %i): %z\n",
-	       errno, STRERROR(errno));
-
-      io_signal_handler(backend, &window_changed,
-			make_winch_handler(self));
-
-      suspend_handle_tty(self->tty_fd);
-    }
   return &self->super;
 }
