@@ -2,7 +2,7 @@
  *
  *
  *
- * $Id: werror.c,v 1.61 2001/07/04 18:37:56 nisse Exp $ */
+ * $Id: werror.c,v 1.63 2001/09/16 19:20:24 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -429,11 +429,11 @@ werror_vformat(const char *f, va_list args)
 	    case 'a':
 	      {
 		int atom = va_arg(args, int);
-		
-		assert(atom);
 
-		werror_write(get_atom_length(atom), get_atom_name(atom));
-		
+		if (atom)
+		  werror_write(get_atom_length(atom), get_atom_name(atom));
+		else
+		  werror_write(9, "<unknown>");
 		break;
 	      }
 	    case 's':
