@@ -2,7 +2,7 @@
  *
  *
  *
- * $Id: xalloc.c,v 1.28 2001/05/02 05:45:39 nisse Exp $ */
+ * $Id: xalloc.c,v 1.29 2001/09/24 21:00:04 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -294,8 +294,9 @@ lsh_object_free(struct lsh_object *o)
 }
 
 #if DEBUG_ALLOC
-struct lsh_object *lsh_object_check(struct lsh_class *class,
-				    struct lsh_object *instance)
+struct lsh_object *
+lsh_object_check(struct lsh_class *class,
+		 struct lsh_object *instance)
 {
   if (!instance)
     return instance;
@@ -308,13 +309,15 @@ struct lsh_object *lsh_object_check(struct lsh_class *class,
 
   if ( (instance->alloc_method == LSH_ALLOC_HEAP)
        && (instance->isa != class))
-    fatal("lsh_object_check: Type error!\n");
+    fatal("lsh_object_check: Type error, expected %z, got %z!\n",
+	  class->name, instance->isa->name);
 
   return instance;
 }
 
-struct lsh_object *lsh_object_check_subtype(struct lsh_class *class,
-					    struct lsh_object *instance)
+struct lsh_object *
+lsh_object_check_subtype(struct lsh_class *class,
+			 struct lsh_object *instance)
 {
   struct lsh_class *type;
   
@@ -343,7 +346,8 @@ struct lsh_object *lsh_object_check_subtype(struct lsh_class *class,
     if (type == class)
       return instance;
 
-  fatal("lsh_object_check_subtype: Type error!\n");
+  fatal("lsh_object_check_subtype: Type error, expected %z, got %z!\n",
+	class->name, instance->isa->name);
 }
 #endif /* DEBUG_ALLOC */
 
